@@ -13,9 +13,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -32,12 +30,11 @@
 #include <math.h>
 #include <webkit2/webkit2.h>
 
-typedef struct
-{
-  char *schema;
-  char *key;
-  char *webkit_pref;
-  void (*callback) (GSettings *settings, char *key, gpointer data);
+typedef struct {
+  const char *schema;
+  const char *key;
+  const char *webkit_pref;
+  void (*callback)(GSettings *settings, const char *key, gpointer data);
 } PrefData;
 
 #define ENABLE_SCRIPTS_SETTING "enable-javascript"
@@ -46,8 +43,8 @@ static WebKitSettings *webkit_settings = NULL;
 
 static void
 user_style_sheet_output_stream_splice_cb (GOutputStream *output_stream,
-                                          GAsyncResult *result,
-                                          gpointer user_data)
+                                          GAsyncResult  *result,
+                                          gpointer       user_data)
 {
   gssize bytes;
 
@@ -65,9 +62,9 @@ user_style_sheet_output_stream_splice_cb (GOutputStream *output_stream,
 }
 
 static void
-user_style_seet_read_cb (GFile *file,
+user_style_seet_read_cb (GFile        *file,
                          GAsyncResult *result,
-                         gpointer user_data)
+                         gpointer      user_data)
 {
   GFileInputStream *input_stream;
   GOutputStream *output_stream;
@@ -89,9 +86,9 @@ user_style_seet_read_cb (GFile *file,
 }
 
 static void
-webkit_pref_callback_user_stylesheet (GSettings *settings,
-                                      char *key,
-                                      gpointer data)
+webkit_pref_callback_user_stylesheet (GSettings  *settings,
+                                      const char *key,
+                                      gpointer    data)
 {
   gboolean value;
 
@@ -121,7 +118,7 @@ webkit_pref_get_vendor_user_agent (void)
 
   branding_keyfile = g_key_file_new ();
 
-  if (g_key_file_load_from_file (branding_keyfile, SHARE_DIR"/branding.conf",
+  if (g_key_file_load_from_file (branding_keyfile, SHARE_DIR "/branding.conf",
                                  G_KEY_FILE_NONE, NULL)) {
     char *vendor;
     char *vendor_sub;
@@ -168,39 +165,44 @@ webkit_pref_get_internal_user_agent (void)
   initialized = TRUE;
 
   vendor_user_agent = webkit_pref_get_vendor_user_agent ();
-  if (!vendor_user_agent)
-    return NULL;
-
   webkit_user_agent = webkit_settings_get_user_agent (webkit_settings);
-  user_agent = g_strdup_printf ("%s %s Epiphany/%s", webkit_user_agent,
-                                vendor_user_agent, VERSION);
-  g_free (vendor_user_agent);
+
+  if (vendor_user_agent) {
+    user_agent = g_strdup_printf ("%s %s Epiphany/%s", webkit_user_agent,
+                                  vendor_user_agent, VERSION);
+    g_free (vendor_user_agent);
+  } else {
+    user_agent = g_strdup_printf ("%s Epiphany/%s", webkit_user_agent, VERSION);
+  }
 
   return user_agent;
 }
 
 static void
-webkit_pref_callback_user_agent (GSettings *settings,
-                                 char *key,
-                                 gpointer data)
+webkit_pref_callback_user_agent (GSettings  *settings,
+                                 const char *key,
+                                 gpointer    data)
 {
+  EphyEmbedShell *shell = ephy_embed_shell_get_default ();
   char *value;
-  const char *internal_user_agent;
+  char *user_agent;
+  const char *base_user_agent;
 
   value = g_settings_get_string (settings, key);
-  if (value != NULL && value[0] != '\0') {
-    webkit_settings_set_user_agent (webkit_settings, value);
-    g_free (value);
-    return;
-  }
-  g_free (value);
-
-  internal_user_agent = webkit_pref_get_internal_user_agent ();
-  if (internal_user_agent)
-    webkit_settings_set_user_agent (webkit_settings, internal_user_agent);
+  if (value != NULL && value[0] != '\0')
+    base_user_agent = value;
   else
-    webkit_settings_set_user_agent_with_application_details (webkit_settings,
-                                                             "Epiphany", VERSION);
+    base_user_agent = webkit_pref_get_internal_user_agent ();
+
+  if (ephy_embed_shell_get_mode (shell) == EPHY_EMBED_SHELL_MODE_APPLICATION)
+    user_agent = g_strdup_printf ("%s (Web App)", base_user_agent);
+  else
+    user_agent = g_strdup (base_user_agent);
+
+  webkit_settings_set_user_agent (webkit_settings, user_agent);
+
+  g_free (user_agent);
+  g_free (value);
 }
 
 static gdouble
@@ -236,9 +238,9 @@ normalize_font_size (gdouble font_size)
 }
 
 static void
-webkit_pref_callback_font_size (GSettings *settings,
-                                char *key,
-                                gpointer data)
+webkit_pref_callback_font_size (GSettings  *settings,
+                                const char *key,
+                                gpointer    data)
 {
   char *webkit_pref = data;
   char *value = NULL;
@@ -250,8 +252,7 @@ webkit_pref_callback_font_size (GSettings *settings,
   /* If we are changing a GNOME font value and we are not using GNOME fonts in
    * Epiphany, return. */
   if (g_strcmp0 (schema, EPHY_PREFS_WEB_SCHEMA) != 0 &&
-      g_settings_get_boolean (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_USE_GNOME_FONTS) != TRUE)
-  {
+      g_settings_get_boolean (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_USE_GNOME_FONTS) != TRUE) {
     g_free (schema);
     return;
   }
@@ -260,7 +261,7 @@ webkit_pref_callback_font_size (GSettings *settings,
   value = g_settings_get_string (settings, key);
 
   if (value) {
-    PangoFontDescription* desc;
+    PangoFontDescription *desc;
 
     desc = pango_font_description_from_string (value);
     size = pango_font_description_get_size (desc);
@@ -274,9 +275,9 @@ webkit_pref_callback_font_size (GSettings *settings,
 }
 
 static void
-webkit_pref_callback_font_family (GSettings *settings,
-                                  char *key,
-                                  gpointer data)
+webkit_pref_callback_font_family (GSettings  *settings,
+                                  const char *key,
+                                  gpointer    data)
 {
   char *webkit_pref = data;
   char *value = NULL;
@@ -287,8 +288,7 @@ webkit_pref_callback_font_family (GSettings *settings,
   /* If we are changing a GNOME font value and we are not using GNOME fonts in
    * Epiphany, return. */
   if (g_strcmp0 (schema, EPHY_PREFS_WEB_SCHEMA) != 0 &&
-      g_settings_get_boolean (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_USE_GNOME_FONTS) != TRUE)
-  {
+      g_settings_get_boolean (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_USE_GNOME_FONTS) != TRUE) {
     g_free (schema);
     return;
   }
@@ -297,7 +297,7 @@ webkit_pref_callback_font_family (GSettings *settings,
   value = g_settings_get_string (settings, key);
 
   if (value) {
-    PangoFontDescription* desc;
+    PangoFontDescription *desc;
     const char *family = NULL;
 
     desc = pango_font_description_from_string (value);
@@ -338,13 +338,13 @@ normalize_languages (char **languages)
 /* Based on Christian Persch's code from gecko backend of epiphany
    (old transform_accept_languages_list() function) */
 static void
-webkit_pref_callback_accept_languages (GSettings *settings,
-                                       char *key,
-                                       gpointer data)
+webkit_pref_callback_accept_languages (GSettings  *settings,
+                                       const char *key,
+                                       gpointer    data)
 {
   GArray *array;
   char **languages;
-  int i;
+  guint i;
   EphyEmbedShell *shell = ephy_embed_shell_get_default ();
   WebKitWebContext *web_context = ephy_embed_shell_get_web_context (shell);
 
@@ -353,12 +353,12 @@ webkit_pref_callback_accept_languages (GSettings *settings,
   array = g_array_new (TRUE, FALSE, sizeof (char *));
 
   for (i = 0; languages[i]; i++) {
-      if (!g_strcmp0 (languages[i], "system")) {
-        ephy_langs_append_languages (array);
-      } else if (languages[i][0] != '\0') {
-        char *str = g_ascii_strdown (languages[i], -1);
-        g_array_append_val (array, str);
-      }
+    if (!g_strcmp0 (languages[i], "system")) {
+      ephy_langs_append_languages (array);
+    } else if (languages[i][0] != '\0') {
+      char *str = g_ascii_strdown (languages[i], -1);
+      g_array_append_val (array, str);
+    }
   }
 
   ephy_langs_sanitise (array);
@@ -372,13 +372,15 @@ webkit_pref_callback_accept_languages (GSettings *settings,
   }
 
   g_strfreev (languages);
+  for (i = 0; i < array->len; i++)
+    g_free (g_array_index (array, char *, i));
   g_array_free (array, TRUE);
 }
 
 
 void
 ephy_embed_prefs_set_cookie_accept_policy (WebKitCookieManager *cookie_manager,
-                                           const char *settings_policy)
+                                           const char          *settings_policy)
 {
   WebKitCookieAcceptPolicy policy;
 
@@ -397,9 +399,9 @@ ephy_embed_prefs_set_cookie_accept_policy (WebKitCookieManager *cookie_manager,
 }
 
 static void
-webkit_pref_callback_cookie_accept_policy (GSettings *settings,
-                                           char *key,
-                                           gpointer data)
+webkit_pref_callback_cookie_accept_policy (GSettings  *settings,
+                                           const char *key,
+                                           gpointer    data)
 {
   WebKitCookieManager *cookie_manager;
   char *value;
@@ -416,7 +418,7 @@ webkit_pref_callback_cookie_accept_policy (GSettings *settings,
 }
 
 static void
-ephy_embed_prefs_update_font_settings (GSettings *ephy_settings, char *key)
+ephy_embed_prefs_update_font_settings (GSettings *ephy_settings, const char *key)
 {
   if (g_settings_get_boolean (ephy_settings, key)) {
     g_object_set (webkit_settings,
@@ -429,33 +431,33 @@ ephy_embed_prefs_update_font_settings (GSettings *ephy_settings, char *key)
   } else {
     /* Sync with Epiphany values */
     webkit_pref_callback_font_size (ephy_settings, EPHY_PREFS_WEB_SERIF_FONT,
-                                    "default-font-size");
+                                    (gpointer)"default-font-size");
     webkit_pref_callback_font_size (ephy_settings, EPHY_PREFS_WEB_MONOSPACE_FONT,
-                                    "default-monospace-font-size");
+                                    (gpointer)"default-monospace-font-size");
 
     webkit_pref_callback_font_family (ephy_settings, EPHY_PREFS_WEB_SERIF_FONT,
-                                      "default-font-family");
+                                      (gpointer)"default-font-family");
     webkit_pref_callback_font_family (ephy_settings, EPHY_PREFS_WEB_SANS_SERIF_FONT,
-                                      "sans-serif-font-family");
+                                      (gpointer)"sans-serif-font-family");
     webkit_pref_callback_font_family (ephy_settings, EPHY_PREFS_WEB_MONOSPACE_FONT,
-                                      "monospace-font-family");
+                                      (gpointer)"monospace-font-family");
     webkit_pref_callback_font_family (ephy_settings, EPHY_PREFS_WEB_SERIF_FONT,
-                                      "serif-font-family");
+                                      (gpointer)"serif-font-family");
   }
 }
 
 static void
-webkit_pref_callback_gnome_fonts (GSettings *ephy_settings,
-                                  char *key,
-                                  gpointer data)
+webkit_pref_callback_gnome_fonts (GSettings  *ephy_settings,
+                                  const char *key,
+                                  gpointer    data)
 {
   ephy_embed_prefs_update_font_settings (ephy_settings, key);
 }
 
 static void
-webkit_pref_callback_enable_spell_checking (GSettings *settings,
-                                            char *key,
-                                            gpointer data)
+webkit_pref_callback_enable_spell_checking (GSettings  *settings,
+                                            const char *key,
+                                            gpointer    data)
 {
   WebKitWebContext *web_context;
   gboolean value = FALSE;
@@ -470,7 +472,7 @@ webkit_pref_callback_enable_spell_checking (GSettings *settings,
     char **languages = g_settings_get_strv (settings, EPHY_PREFS_WEB_LANGUAGE);
     char **normalized = normalize_languages (languages);
 
-    webkit_web_context_set_spell_checking_languages (web_context, (const char* const *)normalized);
+    webkit_web_context_set_spell_checking_languages (web_context, (const char * const *)normalized);
 
     g_strfreev (languages);
     g_strfreev (normalized);
@@ -479,74 +481,74 @@ webkit_pref_callback_enable_spell_checking (GSettings *settings,
 
 static void
 gtk_settings_xft_dpi_changed_cb (GtkSettings *gtk_settings,
-                                 GParamSpec *pspec,
-                                 gpointer data)
+                                 GParamSpec  *pspec,
+                                 gpointer     data)
 {
   GSettings *gsettings = ephy_settings_get (EPHY_PREFS_WEB_SCHEMA);
   ephy_embed_prefs_update_font_settings (gsettings, EPHY_PREFS_WEB_USE_GNOME_FONTS);
 }
 
 static const PrefData webkit_pref_entries[] =
-  {
-    /* Epiphany font settings */
-    { EPHY_PREFS_WEB_SCHEMA,
-      EPHY_PREFS_WEB_SERIF_FONT,
-      "default-font-size",
-      webkit_pref_callback_font_size },
-    { EPHY_PREFS_WEB_SCHEMA,
-      EPHY_PREFS_WEB_MONOSPACE_FONT,
-      "default-monospace-font-size",
-      webkit_pref_callback_font_size },
-    { EPHY_PREFS_WEB_SCHEMA,
-      EPHY_PREFS_WEB_SERIF_FONT,
-      "default-font-family",
-      webkit_pref_callback_font_family },
-    { EPHY_PREFS_WEB_SCHEMA,
-      EPHY_PREFS_WEB_SANS_SERIF_FONT,
-      "sans-serif-font-family",
-      webkit_pref_callback_font_family },
-    { EPHY_PREFS_WEB_SCHEMA,
-      EPHY_PREFS_WEB_MONOSPACE_FONT,
-      "monospace-font-family",
-      webkit_pref_callback_font_family },
-    { EPHY_PREFS_WEB_SCHEMA,
-      EPHY_PREFS_WEB_SERIF_FONT,
-      "serif-font-family",
-      webkit_pref_callback_font_family },
+{
+  /* Epiphany font settings */
+  { EPHY_PREFS_WEB_SCHEMA,
+    EPHY_PREFS_WEB_SERIF_FONT,
+    "default-font-size",
+    webkit_pref_callback_font_size },
+  { EPHY_PREFS_WEB_SCHEMA,
+    EPHY_PREFS_WEB_MONOSPACE_FONT,
+    "default-monospace-font-size",
+    webkit_pref_callback_font_size },
+  { EPHY_PREFS_WEB_SCHEMA,
+    EPHY_PREFS_WEB_SERIF_FONT,
+    "default-font-family",
+    webkit_pref_callback_font_family },
+  { EPHY_PREFS_WEB_SCHEMA,
+    EPHY_PREFS_WEB_SANS_SERIF_FONT,
+    "sans-serif-font-family",
+    webkit_pref_callback_font_family },
+  { EPHY_PREFS_WEB_SCHEMA,
+    EPHY_PREFS_WEB_MONOSPACE_FONT,
+    "monospace-font-family",
+    webkit_pref_callback_font_family },
+  { EPHY_PREFS_WEB_SCHEMA,
+    EPHY_PREFS_WEB_SERIF_FONT,
+    "serif-font-family",
+    webkit_pref_callback_font_family },
 
-    { EPHY_PREFS_WEB_SCHEMA,
-      EPHY_PREFS_WEB_USE_GNOME_FONTS,
-      NULL,
-      webkit_pref_callback_gnome_fonts },
+  { EPHY_PREFS_WEB_SCHEMA,
+    EPHY_PREFS_WEB_USE_GNOME_FONTS,
+    NULL,
+    webkit_pref_callback_gnome_fonts },
 
-    { EPHY_PREFS_WEB_SCHEMA,
-      EPHY_PREFS_WEB_ENABLE_SPELL_CHECKING,
-      NULL,
-      webkit_pref_callback_enable_spell_checking },
+  { EPHY_PREFS_WEB_SCHEMA,
+    EPHY_PREFS_WEB_ENABLE_SPELL_CHECKING,
+    NULL,
+    webkit_pref_callback_enable_spell_checking },
 
-    { EPHY_PREFS_WEB_SCHEMA,
-      EPHY_PREFS_WEB_ENABLE_USER_CSS,
-      "user-stylesheet-uri",
-      webkit_pref_callback_user_stylesheet },
-    { EPHY_PREFS_WEB_SCHEMA,
-      EPHY_PREFS_WEB_LANGUAGE,
-      "accept-language",
-      webkit_pref_callback_accept_languages },
-    { EPHY_PREFS_SCHEMA,
-      EPHY_PREFS_USER_AGENT,
-      "user-agent",
-      webkit_pref_callback_user_agent },
-    { EPHY_PREFS_WEB_SCHEMA,
-      EPHY_PREFS_WEB_COOKIES_POLICY,
-      "accept-policy",
-      webkit_pref_callback_cookie_accept_policy },
-  };
+  { EPHY_PREFS_WEB_SCHEMA,
+    EPHY_PREFS_WEB_ENABLE_USER_CSS,
+    "user-stylesheet-uri",
+    webkit_pref_callback_user_stylesheet },
+  { EPHY_PREFS_WEB_SCHEMA,
+    EPHY_PREFS_WEB_LANGUAGE,
+    "accept-language",
+    webkit_pref_callback_accept_languages },
+  { EPHY_PREFS_SCHEMA,
+    EPHY_PREFS_USER_AGENT,
+    "user-agent",
+    webkit_pref_callback_user_agent },
+  { EPHY_PREFS_WEB_SCHEMA,
+    EPHY_PREFS_WEB_COOKIES_POLICY,
+    "accept-policy",
+    webkit_pref_callback_cookie_accept_policy },
+};
 
 static gpointer
 ephy_embed_prefs_init (gpointer user_data)
 {
   GtkSettings *gtk_settings;
-  int i;
+  guint i;
 
   webkit_settings = webkit_settings_new_with_settings ("enable-developer-extras", TRUE,
                                                        "enable-fullscreen", TRUE,
@@ -564,11 +566,11 @@ ephy_embed_prefs_init (gpointer user_data)
 
     webkit_pref_entries[i].callback (settings,
                                      webkit_pref_entries[i].key,
-                                     webkit_pref_entries[i].webkit_pref);
+                                     (gpointer)webkit_pref_entries[i].webkit_pref);
 
     g_signal_connect (settings, key,
                       G_CALLBACK (webkit_pref_entries[i].callback),
-                      webkit_pref_entries[i].webkit_pref);
+                      (gpointer)webkit_pref_entries[i].webkit_pref);
     g_free (key);
   }
 
