@@ -13,15 +13,9 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#if !defined (__EPHY_EPIPHANY_H_INSIDE__) && !defined (EPIPHANY_COMPILATION)
-#error "Only <epiphany/epiphany.h> can be included directly."
-#endif
-
 #ifndef EPHY_WEB_EXTENSION_PROXY_H
 #define EPHY_WEB_EXTENSION_PROXY_H
 
@@ -29,35 +23,11 @@
 
 G_BEGIN_DECLS
 
-#define EPHY_TYPE_WEB_EXTENSION_PROXY         (ephy_web_extension_proxy_get_type ())
-#define EPHY_WEB_EXTENSION_PROXY(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), EPHY_TYPE_WEB_EXTENSION_PROXY, EphyWebExtensionProxy))
-#define EPHY_IS_WEB_EXTENSION_PROXY(o)        (G_TYPE_CHECK_INSTANCE_TYPE ((o), EPHY_TYPE_WEB_EXTENSION_PROXY))
-#define EPHY_WEB_EXTENSION_PROXY_CLASS(k)     (G_TYPE_CHECK_CLASS_CAST((k), EPHY_TYPE_WEB_EXTENSION_PROXY, EphyWebExtensionProxyClass))
-#define EPHY_IS_WEB_EXTENSION_PROXY_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), EPHY_TYPE_WEB_EXTENSION_PROXY))
-#define EPHY_WEB_EXTENSION_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), EPHY_TYPE_WEB_EXTENSION_PROXY, EphyWebExtensionProxyClass))
+#define EPHY_TYPE_WEB_EXTENSION_PROXY (ephy_web_extension_proxy_get_type ())
 
-typedef struct _EphyWebExtensionProxyClass   EphyWebExtensionProxyClass;
-typedef struct _EphyWebExtensionProxy        EphyWebExtensionProxy;
-typedef struct _EphyWebExtensionProxyPrivate EphyWebExtensionProxyPrivate;
+G_DECLARE_FINAL_TYPE (EphyWebExtensionProxy, ephy_web_extension_proxy, EPHY, WEB_EXTENSION_PROXY, GObject)
 
-struct _EphyWebExtensionProxy
-{
-  GObject parent;
-
-  /*< private >*/
-  EphyWebExtensionProxyPrivate *priv;
-};
-
-struct _EphyWebExtensionProxyClass
-{
-  GObjectClass parent_class;
-};
-
-GType                  ephy_web_extension_proxy_get_type                                  (void);
-
-EphyWebExtensionProxy *ephy_web_extension_proxy_new                                       (GDBusConnection       *bus,
-                                                                                           const char            *service_name);
-const char *           ephy_web_extension_proxy_get_name_owner                            (EphyWebExtensionProxy *web_extension);
+EphyWebExtensionProxy *ephy_web_extension_proxy_new                                       (GDBusConnection       *connection);
 void                   ephy_web_extension_proxy_form_auth_data_save_confirmation_response (EphyWebExtensionProxy *web_extension,
                                                                                            guint                  request_id,
                                                                                            gboolean               response);
@@ -77,7 +47,6 @@ void                   ephy_web_extension_proxy_get_best_web_app_icon
                                                                                            gpointer               user_data);
 gboolean               ephy_web_extension_proxy_get_best_web_app_icon_finish              (EphyWebExtensionProxy *web_extension,
                                                                                            GAsyncResult          *result,
-                                                                                           gboolean              *icon_result,
                                                                                            char                 **icon_uri,
                                                                                            char                 **icon_color,
                                                                                            GError               **error);
