@@ -13,16 +13,15 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
 #include "ephy-embed-form-auth.h"
 
-struct _EphyEmbedFormAuthPrivate
-{
+struct _EphyEmbedFormAuth {
+  GObject parent_instance;
+
   guint64 page_id;
   SoupURI *uri;
   WebKitDOMNode *username_node;
@@ -35,12 +34,12 @@ G_DEFINE_TYPE (EphyEmbedFormAuth, ephy_embed_form_auth, G_TYPE_OBJECT)
 static void
 ephy_embed_form_auth_finalize (GObject *object)
 {
-  EphyEmbedFormAuthPrivate *priv = EPHY_EMBED_FORM_AUTH (object)->priv;
+  EphyEmbedFormAuth *form_auth = EPHY_EMBED_FORM_AUTH (object);
 
-  if (priv->uri)
-    soup_uri_free (priv->uri);
-  g_clear_object (&priv->username_node);
-  g_clear_object (&priv->password_node);
+  if (form_auth->uri)
+    soup_uri_free (form_auth->uri);
+  g_clear_object (&form_auth->username_node);
+  g_clear_object (&form_auth->password_node);
 
   G_OBJECT_CLASS (ephy_embed_form_auth_parent_class)->finalize (object);
 }
@@ -48,7 +47,6 @@ ephy_embed_form_auth_finalize (GObject *object)
 static void
 ephy_embed_form_auth_init (EphyEmbedFormAuth *form_auth)
 {
-  form_auth->priv = G_TYPE_INSTANCE_GET_PRIVATE (form_auth, EPHY_TYPE_EMBED_FORM_AUTH, EphyEmbedFormAuthPrivate);
 }
 
 static void
@@ -57,14 +55,13 @@ ephy_embed_form_auth_class_init (EphyEmbedFormAuthClass *klass)
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
   object_class->finalize = ephy_embed_form_auth_finalize;
-  g_type_class_add_private (object_class, sizeof (EphyEmbedFormAuthPrivate));
 }
 
 EphyEmbedFormAuth *
 ephy_embed_form_auth_new (WebKitWebPage *web_page,
                           WebKitDOMNode *username_node,
                           WebKitDOMNode *password_node,
-                          const char* username)
+                          const char    *username)
 {
   EphyEmbedFormAuth *form_auth;
 
@@ -72,11 +69,11 @@ ephy_embed_form_auth_new (WebKitWebPage *web_page,
 
   form_auth = EPHY_EMBED_FORM_AUTH (g_object_new (EPHY_TYPE_EMBED_FORM_AUTH, NULL));
 
-  form_auth->priv->page_id = webkit_web_page_get_id (web_page);
-  form_auth->priv->uri = soup_uri_new (webkit_web_page_get_uri (web_page));
-  form_auth->priv->username_node = username_node;
-  form_auth->priv->password_node = password_node;
-  form_auth->priv->username = g_strdup (username);
+  form_auth->page_id = webkit_web_page_get_id (web_page);
+  form_auth->uri = soup_uri_new (webkit_web_page_get_uri (web_page));
+  form_auth->username_node = username_node;
+  form_auth->password_node = password_node;
+  form_auth->username = g_strdup (username);
 
   return form_auth;
 }
@@ -84,35 +81,35 @@ ephy_embed_form_auth_new (WebKitWebPage *web_page,
 WebKitDOMNode *
 ephy_embed_form_auth_get_username_node (EphyEmbedFormAuth *form_auth)
 {
-  return form_auth->priv->username_node;
+  return form_auth->username_node;
 }
 
 WebKitDOMNode *
 ephy_embed_form_auth_get_password_node (EphyEmbedFormAuth *form_auth)
 {
-  return form_auth->priv->password_node;
+  return form_auth->password_node;
 }
 
 SoupURI *
 ephy_embed_form_auth_get_uri (EphyEmbedFormAuth *form_auth)
 {
-  return form_auth->priv->uri;
+  return form_auth->uri;
 }
 
 guint64
 ephy_embed_form_auth_get_page_id (EphyEmbedFormAuth *form_auth)
 {
-  return form_auth->priv->page_id;
+  return form_auth->page_id;
 }
 
-const char*
+const char *
 ephy_embed_form_auth_get_username (EphyEmbedFormAuth *form_auth)
 {
-  return form_auth->priv->username;
+  return form_auth->username;
 }
 
 WebKitDOMDocument *
 ephy_embed_form_auth_get_owner_document (EphyEmbedFormAuth *form_auth)
 {
-  return webkit_dom_node_get_owner_document (form_auth->priv->password_node);
+  return webkit_dom_node_get_owner_document (form_auth->password_node);
 }
