@@ -13,9 +13,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -27,8 +25,9 @@
 #include <webkitdom/WebKitDOMElementUnstable.h>
 #include <webkitdom/WebKitDOMDOMTokenList.h>
 
-struct _EphyWebOverviewPrivate
-{
+struct _EphyWebOverview {
+  GObject parent_instance;
+
   WebKitWebPage *web_page;
   EphyWebOverviewModel *model;
   GList *items;
@@ -36,14 +35,16 @@ struct _EphyWebOverviewPrivate
   GHashTable *delayed_thumbnail_changes;
 };
 
-enum
-{
+G_DEFINE_TYPE (EphyWebOverview, ephy_web_overview, G_TYPE_OBJECT)
+
+enum {
   PROP_0,
   PROP_WEB_PAGE,
   PROP_MODEL,
+  LAST_PROP
 };
 
-G_DEFINE_TYPE (EphyWebOverview, ephy_web_overview, G_TYPE_OBJECT)
+static GParamSpec *obj_properties[LAST_PROP];
 
 typedef struct {
   char *url;
@@ -67,7 +68,7 @@ overview_item_new (WebKitDOMElement *anchor)
   nodes = webkit_dom_node_get_child_nodes (WEBKIT_DOM_NODE (anchor));
   n_nodes = webkit_dom_node_list_get_length (nodes);
   for (i = 0; i < n_nodes; i++) {
-    WebKitDOMNode* node = webkit_dom_node_list_item (nodes, i);
+    WebKitDOMNode *node = webkit_dom_node_list_item (nodes, i);
     WebKitDOMElement *element;
     char *tag;
 
@@ -108,7 +109,7 @@ overview_item_free (OverviewItem *item)
 
 static void
 update_thumbnail_element_style (WebKitDOMElement *thumbnail,
-                                const char *path)
+                                const char       *path)
 {
   char *style;
 
@@ -117,18 +118,184 @@ update_thumbnail_element_style (WebKitDOMElement *thumbnail,
   g_free (style);
 }
 
+static gboolean
+apply_delayed_thumbnail_change (gpointer key,
+                                gpointer value,
+                                gpointer user_data)
+{
+  EphyWebOverview *overview = EPHY_WEB_OVERVIEW (user_data);
+  const char *url = key;
+  const char *path = value;
+  GList *l;
+
+  for (l = overview->items; l; l = g_list_next (l)) {
+    OverviewItem *item = (OverviewItem *)l->data;
+
+    if (g_strcmp0 (item->url, url) == 0) {
+      update_thumbnail_element_style (item->thumbnail, path);
+      return TRUE;
+    }
+  }
+
+  return FALSE;
+}
+
+static void
+ephy_web_overview_model_thumbnail_changed (EphyWebOverviewModel *model,
+                                           const char           *url,
+                                           const char           *path,
+                                           EphyWebOverview      *overview)
+{
+  GList *l;
+  for (l = overview->items; l; l = g_list_next (l)) {
+    OverviewItem *item = (OverviewItem *)l->data;
+
+    if (g_strcmp0 (item->url, url) == 0) {
+      update_thumbnail_element_style (item->thumbnail, path);
+      return;
+    }
+  }
+
+  if (!overview->delayed_thumbnail_changes) {
+    overview->delayed_thumbnail_changes = g_hash_table_new_full (g_str_hash,
+                                                                 g_str_equal,
+                                                                 g_free,
+                                                                 g_free);
+  }
+
+  /* We got the thumbnail change request before document-loaded. Save the
+   * request, else we will wind up with an overview showing placeholder icons.
+   * This isn't needed for title and URL changes because EphyAboutHandler is
+   * sure to have those right when creating the overview HTML. But thumbnail
+   * changes can arrive delayed if the snapshot service does not have the right
+   * snapshot on demand.
+   */
+  g_hash_table_insert (overview->delayed_thumbnail_changes, g_strdup (url), g_strdup (path));
+}
+
+static void
+ephy_web_overview_model_title_changed (EphyWebOverviewModel *model,
+                                       const char           *url,
+                                       const char           *title,
+                                       EphyWebOverview      *overview)
+{
+  GList *l;
+
+  for (l = overview->items; l; l = g_list_next (l)) {
+    OverviewItem *item = (OverviewItem *)l->data;
+
+    if (g_strcmp0 (item->url, url) != 0)
+      continue;
+
+    webkit_dom_element_set_attribute (item->anchor, "title", title, NULL);
+    webkit_dom_node_set_text_content (WEBKIT_DOM_NODE (item->title), title, NULL);
+  }
+}
+
+static void
+ephy_web_overview_update_thumbnail_in_model_from_element (EphyWebOverview  *overview,
+                                                          const char       *url,
+                                                          WebKitDOMElement *thumbnail)
+{
+  WebKitDOMCSSStyleDeclaration *style;
+  char *background;
+  char *thumbnail_path;
+
+  style = webkit_dom_element_get_style (thumbnail);
+  if (webkit_dom_css_style_declaration_is_property_implicit (style, "background")) {
+    g_object_unref (style);
+    return;
+  }
+
+  background = webkit_dom_css_style_declaration_get_property_value (style, "background");
+  if (!background) {
+    g_object_unref (style);
+    return;
+  }
+
+  thumbnail_path = g_strrstr (background, "file://");
+  if (thumbnail_path) {
+    char *p;
+
+    thumbnail_path += strlen ("file://");
+    p = g_strrstr (thumbnail_path, ")");
+    if (p) {
+      thumbnail_path = g_strndup (thumbnail_path, strlen (thumbnail_path) - strlen (p));
+      g_signal_handlers_block_by_func (overview->model, G_CALLBACK (ephy_web_overview_model_thumbnail_changed), overview);
+      ephy_web_overview_model_set_url_thumbnail (overview->model, url, thumbnail_path);
+      g_signal_handlers_unblock_by_func (overview->model, G_CALLBACK (ephy_web_overview_model_thumbnail_changed), overview);
+      g_free (thumbnail_path);
+    }
+  } else {
+    const char *path;
+
+    /* Check whether the model was updated while the overview was loading */
+    path = ephy_web_overview_model_get_url_thumbnail (overview->model, url);
+    if (path)
+      update_thumbnail_element_style (thumbnail, path);
+  }
+  g_free (background);
+  g_object_unref (style);
+}
+
+static void
+ephy_web_overview_document_loaded (WebKitWebPage   *web_page,
+                                   EphyWebOverview *overview)
+{
+  WebKitDOMDocument *document;
+  WebKitDOMNodeList *nodes;
+  int i, n_nodes;
+
+  document = webkit_web_page_get_dom_document (web_page);
+  nodes = webkit_dom_document_get_elements_by_tag_name (document, "a");
+  n_nodes = webkit_dom_node_list_get_length (nodes);
+  for (i = 0; i < n_nodes; i++) {
+    WebKitDOMElement *element = WEBKIT_DOM_ELEMENT (webkit_dom_node_list_item (nodes, i));
+    char *class;
+
+    class = webkit_dom_element_get_class_name (element);
+    if (g_strcmp0 (class, "overview-item") == 0) {
+      OverviewItem *item = overview_item_new (element);
+
+      /* URLs and titles are always sent from the UI process, but thumbnails don't,
+       * so update the model with the thumbnail of there's one.
+       */
+      ephy_web_overview_update_thumbnail_in_model_from_element (overview, item->url, item->thumbnail);
+
+      overview->items = g_list_prepend (overview->items, item);
+    }
+    g_free (class);
+  }
+  g_object_unref (nodes);
+  overview->items = g_list_reverse (overview->items);
+
+  if (overview->delayed_thumbnail_changes) {
+    g_hash_table_foreach_remove (overview->delayed_thumbnail_changes,
+                                 apply_delayed_thumbnail_change,
+                                 overview);
+    g_clear_pointer (&overview->delayed_thumbnail_changes, g_hash_table_unref);
+  }
+}
+
 static void
 ephy_web_overview_model_urls_changed (EphyWebOverviewModel *model,
-                                      EphyWebOverview *overview)
+                                      EphyWebOverview      *overview)
 {
   GList *urls;
   GList *l;
   GList *items;
-  OverviewItem* item;
+  OverviewItem *item;
+  WebKitDOMDocument *document;
 
   urls = ephy_web_overview_model_get_urls (model);
+  document = webkit_web_page_get_dom_document (overview->web_page);
 
-  items = overview->priv->items;
+  if (document && !overview->items) {
+    /* We were loaded from page cache. The items already exist in the DOM. */
+    ephy_web_overview_document_loaded (overview->web_page, overview);
+  }
+
+  items = overview->items;
   for (l = urls; l; l = g_list_next (l)) {
     EphyWebOverviewModelItem *url = (EphyWebOverviewModelItem *)l->data;
     const char *thumbnail_path;
@@ -143,7 +310,7 @@ ephy_web_overview_model_urls_changed (EphyWebOverviewModel *model,
       g_free (item->url);
       item->url = g_strdup (url->url);
 
-      class_list = webkit_dom_element_get_class_list (webkit_dom_node_get_parent_element (WEBKIT_DOM_NODE (item->anchor)));
+      class_list = webkit_dom_element_get_class_list (item->anchor);
       if (class_list && webkit_dom_dom_token_list_contains (class_list, "overview-removed"))
         webkit_dom_dom_token_list_remove (class_list, NULL, "overview-removed", NULL);
 
@@ -160,48 +327,7 @@ ephy_web_overview_model_urls_changed (EphyWebOverviewModel *model,
       } else {
         webkit_dom_element_remove_attribute (item->thumbnail, "style");
       }
-    } else {
-      WebKitDOMDocument *document;
-      WebKitDOMElement *item_list, *anchor;
-      WebKitDOMNode *new_node;
-
-      item = g_slice_new0 (OverviewItem);
-      item->url = g_strdup (url->url);
-
-      document = webkit_web_page_get_dom_document (overview->priv->web_page);
-      item_list = webkit_dom_document_get_element_by_id (document, "overview-item-list");
-
-      new_node = WEBKIT_DOM_NODE (webkit_dom_document_create_element (document, "LI", NULL));
-      webkit_dom_node_append_child (WEBKIT_DOM_NODE (item_list), WEBKIT_DOM_NODE (new_node), NULL);
-
-      anchor = webkit_dom_document_create_element (document, "A", NULL);
-      item->anchor = g_object_ref (anchor);
-      webkit_dom_element_set_class_name (WEBKIT_DOM_ELEMENT (anchor), "overview-item");
-      webkit_dom_element_set_attribute (WEBKIT_DOM_ELEMENT (anchor), "href", url->url, NULL);
-      webkit_dom_element_set_attribute (WEBKIT_DOM_ELEMENT (anchor), "title", url->title, NULL);
-      webkit_dom_node_append_child (WEBKIT_DOM_NODE (new_node), WEBKIT_DOM_NODE (anchor), NULL);
-
-      new_node = WEBKIT_DOM_NODE (webkit_dom_document_create_element (document, "DIV", NULL));
-      webkit_dom_element_set_class_name (WEBKIT_DOM_ELEMENT (new_node), "overview-close-button");
-      webkit_dom_element_set_attribute (WEBKIT_DOM_ELEMENT (new_node), "onclick", "removeFromOverview(this.parentNode, event)", NULL);
-      webkit_dom_element_set_attribute (WEBKIT_DOM_ELEMENT (new_node), "title", url->title, NULL);
-      webkit_dom_node_set_text_content (new_node, "✖", NULL);
-      webkit_dom_node_append_child (WEBKIT_DOM_NODE (anchor), new_node, NULL);
-
-      new_node = WEBKIT_DOM_NODE (webkit_dom_document_create_element (document, "SPAN", NULL));
-      item->thumbnail = g_object_ref (new_node);
-      webkit_dom_element_set_class_name (WEBKIT_DOM_ELEMENT (new_node), "overview-thumbnail");
-      if (thumbnail_path)
-        update_thumbnail_element_style (WEBKIT_DOM_ELEMENT (new_node), thumbnail_path);
-      webkit_dom_node_append_child (WEBKIT_DOM_NODE (anchor), new_node, NULL);
-
-      new_node = WEBKIT_DOM_NODE (webkit_dom_document_create_element (document, "SPAN", NULL));
-      item->title = g_object_ref (new_node);
-      webkit_dom_element_set_class_name (WEBKIT_DOM_ELEMENT (new_node), "overview-title");
-      webkit_dom_node_set_text_content (new_node, url->title, NULL);
-      webkit_dom_node_append_child (WEBKIT_DOM_NODE (anchor), new_node, NULL);
-
-      overview->priv->items = g_list_append (overview->priv->items, item);
+      g_clear_object (&class_list);
     }
 
     items = g_list_next (items);
@@ -217,186 +343,30 @@ ephy_web_overview_model_urls_changed (EphyWebOverviewModel *model,
     webkit_dom_node_remove_child (webkit_dom_node_get_parent_node (anchor), anchor, NULL);
 
     overview_item_free (item);
-    overview->priv->items = g_list_delete_link (overview->priv->items, items);
+    overview->items = g_list_delete_link (overview->items, items);
 
     items = next;
   }
 }
 
-static gboolean
-apply_delayed_thumbnail_change (gpointer key,
-                                gpointer value,
-                                gpointer user_data)
-{
-  EphyWebOverview *overview = EPHY_WEB_OVERVIEW (user_data);
-  const char *url = key;
-  const char *path = value;
-  GList *l;
-
-  for (l = overview->priv->items; l; l = g_list_next (l)) {
-    OverviewItem *item = (OverviewItem *)l->data;
-
-    if (g_strcmp0 (item->url, url) == 0) {
-      update_thumbnail_element_style (item->thumbnail, path);
-      return TRUE;
-    }
-  }
-
-  return FALSE;
-}
-
 static void
-ephy_web_overview_model_thumbnail_changed (EphyWebOverviewModel *model,
-                                           const char *url,
-                                           const char *path,
-                                           EphyWebOverview *overview)
-{
-  GList *l;
-
-  for (l = overview->priv->items; l; l = g_list_next (l)) {
-    OverviewItem *item = (OverviewItem *)l->data;
-
-    if (g_strcmp0 (item->url, url) == 0) {
-      update_thumbnail_element_style (item->thumbnail, path);
-      return;
-    }
-  }
-
-  if (!overview->priv->delayed_thumbnail_changes) {
-    overview->priv->delayed_thumbnail_changes = g_hash_table_new_full (g_str_hash,
-                                                                       g_str_equal,
-                                                                       g_free,
-                                                                       g_free);
-  }
-
-  /* We got the thumbnail change request before document-loaded. Save the
-   * request, else we will wind up with an overview showing placeholder icons.
-   * This isn't needed for title and URL changes because EphyAboutHandler is
-   * sure to have those right when creating the overview HTML. But thumbnail
-   * changes can arrive delayed if the snapshot service does not have the right
-   * snapshot on demand.
-   */
-  g_hash_table_insert (overview->priv->delayed_thumbnail_changes, g_strdup (url), g_strdup (path));
-}
-
-static void
-ephy_web_overview_model_title_changed (EphyWebOverviewModel *model,
-                                       const char *url,
-                                       const char *title,
-                                       EphyWebOverview *overview)
-{
-  GList *l;
-
-  for (l = overview->priv->items; l; l = g_list_next (l)) {
-    OverviewItem *item = (OverviewItem *)l->data;
-
-    if (g_strcmp0 (item->url, url) != 0)
-      continue;
-
-    webkit_dom_element_set_attribute (item->anchor, "title", title, NULL);
-    webkit_dom_node_set_text_content (WEBKIT_DOM_NODE (item->title), title, NULL);
-  }
-}
-
-static void
-ephy_web_overview_update_thumbnail_in_model_from_element (EphyWebOverview *overview,
-                                                          const char *url,
-                                                          WebKitDOMElement *thumbnail)
-{
-  WebKitDOMCSSStyleDeclaration *style;
-  char *background;
-  char *thumbnail_path;
-
-  style = webkit_dom_element_get_style (thumbnail);
-  if (webkit_dom_css_style_declaration_is_property_implicit (style, "background"))
-    return;
-
-  background = webkit_dom_css_style_declaration_get_property_value (style, "background");
-  if (!background)
-    return;
-
-  thumbnail_path = g_strrstr (background, "file://");
-  if (thumbnail_path) {
-    char *p;
-
-    thumbnail_path += strlen ("file://");
-    p = g_strrstr (thumbnail_path, ")");
-    if (p) {
-      thumbnail_path = g_strndup (thumbnail_path, strlen (thumbnail_path) - strlen (p));
-      g_signal_handlers_block_by_func (overview->priv->model, G_CALLBACK (ephy_web_overview_model_thumbnail_changed), overview);
-      ephy_web_overview_model_set_url_thumbnail (overview->priv->model, url, thumbnail_path);
-      g_signal_handlers_unblock_by_func (overview->priv->model, G_CALLBACK (ephy_web_overview_model_thumbnail_changed), overview);
-      g_free (thumbnail_path);
-    }
-  } else {
-    const char *path;
-
-    /* Check whether the model was updated while the overview was loading */
-    path = ephy_web_overview_model_get_url_thumbnail (overview->priv->model, url);
-    if (path)
-      update_thumbnail_element_style (thumbnail, path);
-  }
-  g_free (background);
-}
-
-static void
-ephy_web_overview_document_loaded (WebKitWebPage *web_page,
-                                   EphyWebOverview *overview)
-{
-  WebKitDOMDocument *document;
-  WebKitDOMNodeList *nodes;
-  int i, n_nodes;
-
-  document = webkit_web_page_get_dom_document (web_page);
-  nodes = webkit_dom_document_get_elements_by_tag_name (document, "a");
-  n_nodes = webkit_dom_node_list_get_length (nodes);
-  for (i = 0; i < n_nodes; i++) {
-    WebKitDOMElement* element = WEBKIT_DOM_ELEMENT (webkit_dom_node_list_item (nodes, i));
-    char *class;
-
-    class = webkit_dom_element_get_class_name (element);
-    if (g_strcmp0 (class, "overview-item") == 0) {
-      OverviewItem *item = overview_item_new (element);
-
-      /* URLs and titles are always sent from the UI process, but thumbnails don't,
-       * so update the model with the thumbnail of there's one.
-       */
-      ephy_web_overview_update_thumbnail_in_model_from_element (overview, item->url, item->thumbnail);
-
-      overview->priv->items = g_list_prepend (overview->priv->items, item);
-    }
-    g_free (class);
-  }
-  g_object_unref (nodes);
-  overview->priv->items = g_list_reverse (overview->priv->items);
-
-  if (overview->priv->delayed_thumbnail_changes) {
-    g_hash_table_foreach_remove (overview->priv->delayed_thumbnail_changes,
-                                 apply_delayed_thumbnail_change,
-                                 overview);
-    g_clear_pointer (&overview->priv->delayed_thumbnail_changes, g_hash_table_unref);
-  }
-}
-
-static void
-ephy_web_overview_set_property (GObject *object,
-                                guint prop_id,
+ephy_web_overview_set_property (GObject      *object,
+                                guint         prop_id,
                                 const GValue *value,
-                                GParamSpec *pspec)
+                                GParamSpec   *pspec)
 {
   EphyWebOverview *overview = EPHY_WEB_OVERVIEW (object);
 
-  switch (prop_id)
-  {
-  case PROP_WEB_PAGE:
-    overview->priv->web_page = g_value_get_object (value);
-    break;
-  case PROP_MODEL:
-    overview->priv->model = g_value_get_object (value);
-    break;
-  default:
-    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-    break;
+  switch (prop_id) {
+    case PROP_WEB_PAGE:
+      overview->web_page = g_value_get_object (value);
+      break;
+    case PROP_MODEL:
+      overview->model = g_value_get_object (value);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
   }
 }
 
@@ -405,12 +375,12 @@ ephy_web_overview_dispose (GObject *object)
 {
   EphyWebOverview *overview = EPHY_WEB_OVERVIEW (object);
 
-  if (overview->priv->items) {
-    g_list_free_full (overview->priv->items, (GDestroyNotify)overview_item_free);
-    overview->priv->items = NULL;
+  if (overview->items) {
+    g_list_free_full (overview->items, (GDestroyNotify)overview_item_free);
+    overview->items = NULL;
   }
 
-  g_clear_pointer (&overview->priv->delayed_thumbnail_changes, g_hash_table_unref);
+  g_clear_pointer (&overview->delayed_thumbnail_changes, g_hash_table_unref);
 
   G_OBJECT_CLASS (ephy_web_overview_parent_class)->dispose (object);
 }
@@ -422,17 +392,17 @@ ephy_web_overview_constructed (GObject *object)
 
   G_OBJECT_CLASS (ephy_web_overview_parent_class)->constructed (object);
 
-  g_signal_connect_object (overview->priv->web_page, "document-loaded",
+  g_signal_connect_object (overview->web_page, "document-loaded",
                            G_CALLBACK (ephy_web_overview_document_loaded),
                            overview, 0);
 
-  g_signal_connect_object (overview->priv->model, "urls-changed",
+  g_signal_connect_object (overview->model, "urls-changed",
                            G_CALLBACK (ephy_web_overview_model_urls_changed),
                            overview, 0);
-  g_signal_connect_object (overview->priv->model, "thumbnail-changed",
+  g_signal_connect_object (overview->model, "thumbnail-changed",
                            G_CALLBACK (ephy_web_overview_model_thumbnail_changed),
                            overview, 0);
-  g_signal_connect_object (overview->priv->model, "title-changed",
+  g_signal_connect_object (overview->model, "title-changed",
                            G_CALLBACK (ephy_web_overview_model_title_changed),
                            overview, 0);
 }
@@ -446,36 +416,30 @@ ephy_web_overview_class_init (EphyWebOverviewClass *klass)
   object_class->constructed = ephy_web_overview_constructed;
   object_class->set_property = ephy_web_overview_set_property;
 
-  g_object_class_install_property (object_class,
-                                   PROP_WEB_PAGE,
-                                   g_param_spec_object ("web-page",
-                                                        "WebPage",
-                                                        "The overview WebPage",
-                                                        WEBKIT_TYPE_WEB_PAGE,
-                                                        G_PARAM_WRITABLE |
-                                                        G_PARAM_CONSTRUCT_ONLY |
-                                                        G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
-  g_object_class_install_property (object_class,
-                                   PROP_MODEL,
-                                   g_param_spec_object ("model",
-                                                        "Model",
-                                                        "The overview model",
-                                                        EPHY_TYPE_WEB_OVERVIEW_MODEL,
-                                                        G_PARAM_WRITABLE |
-                                                        G_PARAM_CONSTRUCT_ONLY |
-                                                        G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+  obj_properties[PROP_WEB_PAGE] =
+    g_param_spec_object ("web-page",
+                         "WebPage",
+                         "The overview WebPage",
+                         WEBKIT_TYPE_WEB_PAGE,
+                         G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
 
-  g_type_class_add_private (object_class, sizeof(EphyWebOverviewPrivate));
+  obj_properties[PROP_MODEL] =
+    g_param_spec_object ("model",
+                         "Model",
+                         "The overview model",
+                         EPHY_TYPE_WEB_OVERVIEW_MODEL,
+                         G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS);
+
+  g_object_class_install_properties (object_class, LAST_PROP, obj_properties);
 }
 
 static void
 ephy_web_overview_init (EphyWebOverview *overview)
 {
-  overview->priv = G_TYPE_INSTANCE_GET_PRIVATE (overview, EPHY_TYPE_WEB_OVERVIEW, EphyWebOverviewPrivate);
 }
 
 EphyWebOverview *
-ephy_web_overview_new (WebKitWebPage *web_page,
+ephy_web_overview_new (WebKitWebPage        *web_page,
                        EphyWebOverviewModel *model)
 {
   g_return_val_if_fail (WEBKIT_IS_WEB_PAGE (web_page), NULL);
