@@ -14,9 +14,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -45,7 +43,7 @@ ephy_profile_utils_get_migration_version_for_profile_dir (const char *profile_di
     g_file_get_contents (migrated_file, &contents, &size, NULL);
 
     if (contents != NULL)
-      result = sscanf(contents, "%d", &latest);
+      result = sscanf (contents, "%d", &latest);
 
     g_free (contents);
 
@@ -104,7 +102,7 @@ ephy_profile_utils_do_migration (const char *profile_directory, int test_to_run,
   GError *error = NULL;
   char *index = NULL, *version = NULL;
   int status;
-  char *argv[6] = { EPHY_PROFILE_MIGRATOR, "-v" };
+  const char *argv[6] = { EPHY_PROFILE_MIGRATOR, "-v" };
   int i = 2; /* index for argv, start filling at 2. */
   char **envp;
 
@@ -137,15 +135,15 @@ ephy_profile_utils_do_migration (const char *profile_directory, int test_to_run,
   argv[i++] = NULL;
 
   if (debug)
-    argv[0] = ABS_TOP_BUILD_DIR"/lib/"EPHY_PROFILE_MIGRATOR;
+    argv[0] = ABS_TOP_BUILD_DIR "/lib/"EPHY_PROFILE_MIGRATOR;
 
-  ret = g_spawn_sync (NULL, argv, envp, G_SPAWN_SEARCH_PATH,
+  ret = g_spawn_sync (NULL, (char **)argv, envp, G_SPAWN_SEARCH_PATH,
                       NULL, NULL, NULL, NULL,
                       &status, &error);
   g_free (index);
   g_free (version);
   g_strfreev (envp);
-    
+
   if (error) {
     LOG ("Failed to run migrator: %s", error->message);
     g_error_free (error);
