@@ -13,8 +13,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -26,13 +25,14 @@
 #include <stdio.h>
 #include <string.h>
 
-G_DEFINE_TYPE (EphySMaps, ephy_smaps, G_TYPE_OBJECT)
-
-struct _EphySMapsPrivate {
+struct _EphySMaps {
+  GObject parent_instance;
   GRegex *header;
   GRegex *detail;
 };
 
+G_DEFINE_TYPE (EphySMaps, ephy_smaps, G_TYPE_OBJECT)
+
 typedef struct {
   char *start;
   char *end;
@@ -69,20 +69,21 @@ typedef enum {
 static const char *get_ephy_process_name (EphyProcess process)
 {
   switch (process) {
-  case EPHY_PROCESS_EPIPHANY:
-    return "Browser";
-  case EPHY_PROCESS_WEB:
-    return "Web Process";
-  case EPHY_PROCESS_PLUGIN:
-    return "Plugin Process";
-  case EPHY_PROCESS_OTHER:
-    g_assert_not_reached ();
+    case EPHY_PROCESS_EPIPHANY:
+      return "Browser";
+    case EPHY_PROCESS_WEB:
+      return "Web Process";
+    case EPHY_PROCESS_PLUGIN:
+      return "Plugin Process";
+    case EPHY_PROCESS_OTHER:
+    default:
+      g_assert_not_reached ();
   }
 
   return NULL;
 }
 
-static void vma_free (VMA_t* vma)
+static void vma_free (VMA_t *vma)
 {
   g_free (vma->start);
   g_free (vma->end);
@@ -198,7 +199,6 @@ static void ephy_smaps_pid_to_html (EphySMaps *smaps, GString *str, pid_t pid, E
   VMA_t *vma = NULL;
   GHashTable *anon_hash, *mapped_hash;
   GSList *vma_entries = NULL, *p;
-  EphySMapsPrivate *priv = smaps->priv;
 
   path = g_strdup_printf ("/proc/%u/smaps", pid);
   file = g_file_new_for_path (path);
@@ -207,7 +207,7 @@ static void ephy_smaps_pid_to_html (EphySMaps *smaps, GString *str, pid_t pid, E
   stream = g_file_read (file, NULL, &error);
   g_object_unref (file);
 
-  if (error && error->code == G_IO_ERROR_NOT_FOUND ) {
+  if (error && error->code == G_IO_ERROR_NOT_FOUND) {
     /* This is not GNU/Linux, do nothing. */
     g_error_free (error);
     return;
@@ -220,7 +220,7 @@ static void ephy_smaps_pid_to_html (EphySMaps *smaps, GString *str, pid_t pid, E
     GMatchInfo *match_info = NULL;
     gboolean matched = FALSE;
 
-    g_regex_match (priv->header, line, 0, &match_info);
+    g_regex_match (smaps->header, line, 0, &match_info);
     if (g_match_info_matches (match_info)) {
       matched = TRUE;
 
@@ -238,13 +238,13 @@ static void ephy_smaps_pid_to_html (EphySMaps *smaps, GString *str, pid_t pid, E
       vma->inode = g_match_info_fetch (match_info, 7);
       vma->filename = g_match_info_fetch (match_info, 8);
     }
-    
+
     g_match_info_free (match_info);
 
     if (matched)
       goto out;
 
-    g_regex_match (priv->detail, line, 0, &match_info);
+    g_regex_match (smaps->detail, line, 0, &match_info);
     if (g_match_info_matches (match_info)) {
       char *name = g_match_info_fetch (match_info, 1);
       char **size = NULL;
@@ -271,7 +271,7 @@ static void ephy_smaps_pid_to_html (EphySMaps *smaps, GString *str, pid_t pid, E
     }
 
     g_match_info_free (match_info);
-  out:
+ out:
     g_free (line);
   }
 
@@ -292,7 +292,7 @@ static void ephy_smaps_pid_to_html (EphySMaps *smaps, GString *str, pid_t pid, E
                                        (GDestroyNotify)perm_entry_free);
 
   for (p = vma_entries; p; p = p->next) {
-    VMA_t *entry = (VMA_t*)p->data;
+    VMA_t *entry = (VMA_t *)p->data;
 
     if (g_strcmp0 (entry->major, "00") && g_strcmp0 (entry->minor, "00"))
       add_to_perm_entry (anon_hash, entry);
@@ -438,7 +438,7 @@ static void ephy_smaps_pid_children_to_html (EphySMaps *smaps, GString *str, pid
   g_dir_close (proc);
 }
 
-char* ephy_smaps_to_html (EphySMaps *smaps)
+char *ephy_smaps_to_html (EphySMaps *smaps)
 {
   GString *str = g_string_new ("");
   pid_t pid = getpid ();
@@ -456,23 +456,21 @@ char* ephy_smaps_to_html (EphySMaps *smaps)
 static void
 ephy_smaps_init (EphySMaps *smaps)
 {
-  smaps->priv = G_TYPE_INSTANCE_GET_PRIVATE (smaps, EPHY_TYPE_SMAPS, EphySMapsPrivate);
-
   /* Prepare the regexps for the smaps file. */
-  smaps->priv->header = g_regex_new ("^([0-9a-f]+)-([0-9a-f]+) (....) ([0-9a-f]+) (..):(..) (\\d+) *(.*)$",
-                                     G_REGEX_OPTIMIZE,
-                                     0,
-                                     NULL);
-  smaps->priv->detail = g_regex_new ("^(.*): +(\\d+) kB", G_REGEX_OPTIMIZE, 0, NULL);
+  smaps->header = g_regex_new ("^([0-9a-f]+)-([0-9a-f]+) (....) ([0-9a-f]+) (..):(..) (\\d+) *(.*)$",
+                               G_REGEX_OPTIMIZE,
+                               0,
+                               NULL);
+  smaps->detail = g_regex_new ("^(.*): +(\\d+) kB", G_REGEX_OPTIMIZE, 0, NULL);
 }
 
 static void
 ephy_smaps_finalize (GObject *obj)
 {
-  EphySMapsPrivate *priv = EPHY_SMAPS (obj)->priv;
+  EphySMaps *smaps = EPHY_SMAPS (obj);
 
-  g_regex_unref (priv->header);
-  g_regex_unref (priv->detail);
+  g_regex_unref (smaps->header);
+  g_regex_unref (smaps->detail);
 
   G_OBJECT_CLASS (ephy_smaps_parent_class)->finalize (obj);
 }
@@ -483,11 +481,9 @@ ephy_smaps_class_init (EphySMapsClass *smaps_class)
   GObjectClass *gobject_class = G_OBJECT_CLASS (smaps_class);
 
   gobject_class->finalize = ephy_smaps_finalize;
-
-  g_type_class_add_private (smaps_class, sizeof (EphySMapsPrivate));
 }
 
-EphySMaps *ephy_smaps_new ()
+EphySMaps *ephy_smaps_new (void)
 {
   return EPHY_SMAPS (g_object_new (EPHY_TYPE_SMAPS, NULL));
 }
