@@ -13,9 +13,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -29,10 +27,9 @@
 #include <libgnome-desktop/gnome-desktop-thumbnail.h>
 #include <webkit2/webkit2.h>
 
-#define EPHY_SNAPSHOT_SERVICE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), EPHY_TYPE_SNAPSHOT_SERVICE, EphySnapshotServicePrivate))
+struct _EphySnapshotService {
+  GObject parent_instance;
 
-struct _EphySnapshotServicePrivate
-{
   /* Disk cache */
   GnomeDesktopThumbnailFactory *factory;
 
@@ -71,8 +68,6 @@ ephy_snapshot_service_class_init (EphySnapshotServiceClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-  g_type_class_add_private (klass, sizeof (EphySnapshotServicePrivate));
-
   /**
    * EphySnapshotService::snapshot-saved:
    * @url: the URL the snapshot was saved for
@@ -94,11 +89,10 @@ ephy_snapshot_service_class_init (EphySnapshotServiceClass *klass)
 static void
 ephy_snapshot_service_init (EphySnapshotService *self)
 {
-  self->priv = EPHY_SNAPSHOT_SERVICE_GET_PRIVATE (self);
-  self->priv->factory = gnome_desktop_thumbnail_factory_new (GNOME_DESKTOP_THUMBNAIL_SIZE_LARGE);
-  self->priv->cache = g_hash_table_new_full (g_str_hash, g_str_equal,
-                                             (GDestroyNotify)g_free,
-                                             (GDestroyNotify)snapshot_path_cached_data_free);
+  self->factory = gnome_desktop_thumbnail_factory_new (GNOME_DESKTOP_THUMBNAIL_SIZE_LARGE);
+  self->cache = g_hash_table_new_full (g_str_hash, g_str_equal,
+                                       (GDestroyNotify)g_free,
+                                       (GDestroyNotify)snapshot_path_cached_data_free);
 }
 
 static GdkPixbuf *
@@ -146,10 +140,10 @@ ephy_snapshot_service_prepare_snapshot (cairo_surface_t *surface,
 
   g_object_unref (snapshot);
 
+  x_offset = 6;
   if (favicon) {
-    GdkPixbuf* fav_pixbuf;
+    GdkPixbuf *fav_pixbuf;
     int favicon_size = 16;
-    int x_offset = 6;
     int y_offset = gdk_pixbuf_get_height (scaled) - favicon_size - x_offset;
 
     fav_pixbuf = ephy_pixbuf_get_from_surface_scaled (favicon, favicon_size, favicon_size);
@@ -164,72 +158,174 @@ ephy_snapshot_service_prepare_snapshot (cairo_surface_t *surface,
 }
 
 typedef struct {
+  EphySnapshotService *service;
+  GdkPixbuf *snapshot;
   WebKitWebView *web_view;
   time_t mtime;
-  gboolean for_snapshot;
-
-  GdkPixbuf *snapshot;
-  char *path;
+  char *url;
 } SnapshotAsyncData;
 
 static SnapshotAsyncData *
-snapshot_async_data_new (WebKitWebView *web_view,
-                         time_t mtime)
+snapshot_async_data_new (EphySnapshotService *service,
+                         GdkPixbuf           *snapshot,
+                         WebKitWebView       *web_view,
+                         time_t               mtime,
+                         const char          *url)
 {
   SnapshotAsyncData *data;
 
   data = g_slice_new0 (SnapshotAsyncData);
+  data->service = g_object_ref (service);
+  data->snapshot = snapshot ? g_object_ref (snapshot) : NULL;
   data->web_view = web_view;
   data->mtime = mtime;
+  data->url = g_strdup (url);
 
-  g_object_add_weak_pointer (G_OBJECT (web_view), (gpointer *)&data->web_view);
+  if (web_view)
+    g_object_add_weak_pointer (G_OBJECT (web_view), (gpointer *)&data->web_view);
 
   return data;
 }
 
 static SnapshotAsyncData *
-snapshot_async_data_new_for_snapshot (WebKitWebView *web_view,
-                                      time_t mtime)
+snapshot_async_data_copy (SnapshotAsyncData *data)
 {
-  SnapshotAsyncData *data = snapshot_async_data_new (web_view, mtime);
-
-  data->for_snapshot = TRUE;
-
-  return data;
+  SnapshotAsyncData *copy = snapshot_async_data_new (data->service,
+                                                     data->snapshot,
+                                                     data->web_view,
+                                                     data->mtime,
+                                                     data->url);
+  return copy;
 }
 
 static void
 snapshot_async_data_free (SnapshotAsyncData *data)
 {
+  g_clear_object (&data->service);
+  g_clear_object (&data->snapshot);
+
   if (data->web_view)
     g_object_remove_weak_pointer (G_OBJECT (data->web_view), (gpointer *)&data->web_view);
-  g_clear_object(&data->snapshot);
-  g_free (data->path);
+
+  g_free (data->url);
 
   g_slice_free (SnapshotAsyncData, data);
 }
 
+typedef struct {
+  GHashTable *cache;
+  char *url;
+  SnapshotPathCachedData *data;
+} CacheData;
+
+static gboolean
+idle_cache_snapshot_path (gpointer user_data)
+{
+  CacheData *data = (CacheData *)user_data;
+  g_hash_table_insert (data->cache, data->url, data->data);
+  g_hash_table_unref (data->cache);
+  g_free (data);
+
+  return G_SOURCE_REMOVE;
+}
+
+static void
+cache_snapshot_data_in_idle (EphySnapshotService  *service,
+                             const char           *url,
+                             const char           *path,
+                             EphySnapshotFreshness freshness)
+{
+  CacheData *data;
+  data = g_new (CacheData, 1);
+  data->cache = g_hash_table_ref (service->cache);
+  data->url = g_strdup (url);
+  data->data = g_new (SnapshotPathCachedData, 1);
+  data->data->path = g_strdup (path);
+  data->data->freshness = freshness;
+  g_idle_add (idle_cache_snapshot_path, data);
+}
+
+static gboolean
+idle_emit_snapshot_saved (gpointer user_data)
+{
+  SnapshotAsyncData *data = (SnapshotAsyncData *)user_data;
+
+  g_signal_emit (data->service, signals[SNAPSHOT_SAVED], 0, data->url, data->mtime);
+
+  snapshot_async_data_free (data);
+  return G_SOURCE_REMOVE;
+}
+
+static void
+save_snapshot_thread (GTask               *task,
+                      EphySnapshotService *service,
+                      SnapshotAsyncData   *data,
+                      GCancellable        *cancellable)
+{
+  char *path;
+
+  gnome_desktop_thumbnail_factory_save_thumbnail (service->factory,
+                                                  data->snapshot,
+                                                  data->url,
+                                                  data->mtime);
+  g_idle_add (idle_emit_snapshot_saved, snapshot_async_data_copy (data));
+
+  path = gnome_desktop_thumbnail_path_for_uri (data->url, GNOME_DESKTOP_THUMBNAIL_SIZE_LARGE);
+  cache_snapshot_data_in_idle (service, data->url, path, SNAPSHOT_FRESH);
+
+  g_task_return_pointer (task, path, g_free);
+}
+
+static void
+ephy_snapshot_service_save_snapshot_async (EphySnapshotService *service,
+                                           GdkPixbuf           *snapshot,
+                                           const char          *url,
+                                           time_t               mtime,
+                                           GCancellable        *cancellable,
+                                           GAsyncReadyCallback  callback,
+                                           gpointer             user_data)
+{
+  GTask *task;
+
+  g_return_if_fail (EPHY_IS_SNAPSHOT_SERVICE (service));
+  g_return_if_fail (GDK_IS_PIXBUF (snapshot));
+  g_return_if_fail (url != NULL);
+
+  task = g_task_new (service, cancellable, callback, user_data);
+  g_task_set_priority (task, G_PRIORITY_LOW);
+  g_task_set_task_data (task,
+                        snapshot_async_data_new (service, snapshot, NULL, mtime, url),
+                        (GDestroyNotify)snapshot_async_data_free);
+  g_task_run_in_thread (task, (GTaskThreadFunc)save_snapshot_thread);
+  g_object_unref (task);
+}
+
+static char *
+ephy_snapshot_service_save_snapshot_finish (EphySnapshotService *service,
+                                            GAsyncResult        *result,
+                                            GError             **error)
+{
+  g_return_val_if_fail (g_task_is_valid (result, service), NULL);
+
+  return g_task_propagate_pointer (G_TASK (result), error);
+}
+
+
 static void
 snapshot_saved (EphySnapshotService *service,
-                GAsyncResult *result,
-                GTask *task)
+                GAsyncResult        *result,
+                GTask               *task)
 {
-  SnapshotAsyncData *data = g_task_get_task_data (task);
   char *path;
 
   path = ephy_snapshot_service_save_snapshot_finish (service, result, NULL);
-  if (data->for_snapshot) {
-    data->path = path;
-    g_task_return_pointer (task, g_object_ref (data->snapshot), g_object_unref);
-  } else {
-    g_task_return_pointer (task, path, g_free);
-  }
+  g_task_return_pointer (task, path, g_free);
   g_object_unref (task);
 }
 
 static void
 save_snapshot (cairo_surface_t *surface,
-               GTask *task)
+               GTask           *task)
 {
   SnapshotAsyncData *data = g_task_get_task_data (task);
 
@@ -247,8 +343,8 @@ save_snapshot (cairo_surface_t *surface,
 
 static void
 on_snapshot_ready (WebKitWebView *web_view,
-                   GAsyncResult *result,
-                   GTask *task)
+                   GAsyncResult  *result,
+                   GTask         *task)
 {
   cairo_surface_t *surface;
   GError *error = NULL;
@@ -289,7 +385,7 @@ retrieve_snapshot_from_web_view (GTask *task)
 
 static void
 webview_destroyed_cb (GtkWidget *web_view,
-                      GTask *task)
+                      GTask     *task)
 {
   g_task_return_new_error (task,
                            EPHY_SNAPSHOT_SERVICE_ERROR,
@@ -299,16 +395,16 @@ webview_destroyed_cb (GtkWidget *web_view,
 }
 
 static void
-webview_load_changed_cb (WebKitWebView *web_view,
+webview_load_changed_cb (WebKitWebView  *web_view,
                          WebKitLoadEvent load_event,
-                         GTask *task)
+                         GTask          *task)
 {
   if (load_event != WEBKIT_LOAD_FINISHED)
     return;
 
   /* Load finished doesn't ensure that we actually have visible content yet,
      so hold a bit before retrieving the snapshot. */
-  g_idle_add ((GSourceFunc) retrieve_snapshot_from_web_view, task);
+  g_idle_add ((GSourceFunc)retrieve_snapshot_from_web_view, task);
 
   /* Some pages might end up causing this condition to happen twice, so remove
      the handler in order to avoid calling the above idle function twice. */
@@ -317,11 +413,11 @@ webview_load_changed_cb (WebKitWebView *web_view,
 }
 
 static gboolean
-webview_load_failed_cb (WebKitWebView *web_view,
+webview_load_failed_cb (WebKitWebView  *web_view,
                         WebKitLoadEvent load_event,
-                        const char failing_uri,
-                        GError *error,
-                        GTask *task)
+                        const char      failing_uri,
+                        GError         *error,
+                        GTask          *task)
 {
   g_signal_handlers_disconnect_by_func (web_view, webview_load_changed_cb, task);
   g_signal_handlers_disconnect_by_func (web_view, webview_load_failed_cb, task);
@@ -393,14 +489,14 @@ ephy_snapshot_service_get_default (void)
 }
 
 const char *
-ephy_snapshot_service_lookup_snapshot_path (EphySnapshotService *service,
-                                            const char          *url)
+ephy_snapshot_service_lookup_cached_snapshot_path (EphySnapshotService *service,
+                                                   const char          *url)
 {
   SnapshotPathCachedData *data;
 
   g_return_val_if_fail (EPHY_IS_SNAPSHOT_SERVICE (service), NULL);
 
-  data = g_hash_table_lookup (service->priv->cache, url);
+  data = g_hash_table_lookup (service->cache, url);
 
   return data == NULL ? NULL : data->path;
 }
@@ -411,427 +507,25 @@ ephy_snapshot_service_lookup_snapshot_freshness (EphySnapshotService *service,
 {
   SnapshotPathCachedData *data;
 
-  data = g_hash_table_lookup (service->priv->cache, url);
+  data = g_hash_table_lookup (service->cache, url);
 
   return data == NULL ? SNAPSHOT_STALE : data->freshness;
 }
 
-typedef struct {
-  char *url;
-  time_t mtime;
-
-  char *path;
-} SnapshotForURLAsyncData;
-
-static SnapshotForURLAsyncData *
-snapshot_for_url_async_data_new (const char *url,
-                                 time_t mtime)
-{
-  SnapshotForURLAsyncData *data;
-
-  data = g_slice_new0 (SnapshotForURLAsyncData);
-  data->url = g_strdup (url);
-  data->mtime = mtime;
-
-  return data;
-}
-
 static void
-snapshot_for_url_async_data_free (SnapshotForURLAsyncData *data)
-{
-  g_free (data->url);
-  g_free (data->path);
-
-  g_slice_free (SnapshotForURLAsyncData, data);
-}
-
-typedef struct {
-  GHashTable *cache;
-  char *url;
-  SnapshotPathCachedData *data;
-} CacheData;
-
-static gboolean
-idle_cache_snapshot_path (gpointer user_data)
-{
-  CacheData *data = (CacheData *)user_data;
-  g_hash_table_insert (data->cache, data->url, data->data);
-  g_hash_table_unref (data->cache);
-  g_free (data);
-
-  return G_SOURCE_REMOVE;
-}
-
-static void
-cache_snapshot_data_in_idle (EphySnapshotService  *service,
-                             const char           *url,
-                             const char           *path,
-                             EphySnapshotFreshness freshness)
-{
-  CacheData *data;
-  data = g_new (CacheData, 1);
-  data->cache = g_hash_table_ref (service->priv->cache);
-  data->url = g_strdup (url);
-  data->data = g_new (SnapshotPathCachedData, 1);
-  data->data->path = g_strdup (path);
-  data->data->freshness = freshness;
-  g_idle_add (idle_cache_snapshot_path, data);
-}
-
-static void
-get_snapshot_for_url_thread (GTask *task,
-                             EphySnapshotService *service,
-                             SnapshotForURLAsyncData *data,
-                             GCancellable *cancellable)
-{
-  GdkPixbuf *snapshot;
-  GError *error = NULL;
-
-  data->path = gnome_desktop_thumbnail_factory_lookup (service->priv->factory, data->url, data->mtime);
-  if (data->path == NULL) {
-    g_task_return_new_error (task,
-                             EPHY_SNAPSHOT_SERVICE_ERROR,
-                             EPHY_SNAPSHOT_SERVICE_ERROR_NOT_FOUND,
-                             "Snapshot for url \"%s\" not found in cache", data->url);
-    return;
-  }
-
-  cache_snapshot_data_in_idle (service, data->url, data->path, SNAPSHOT_STALE);
-
-  snapshot = gdk_pixbuf_new_from_file (data->path, &error);
-  if (snapshot == NULL) {
-    g_task_return_new_error (task,
-                             EPHY_SNAPSHOT_SERVICE_ERROR,
-                             EPHY_SNAPSHOT_SERVICE_ERROR_INVALID,
-                             "Error creating pixbuf for snapshot file \"%s\": %s",
-                             data->path, error->message);
-    g_error_free (error);
-  }
-
-  g_task_return_pointer (task, snapshot, g_object_unref);
-}
-
-/**
- * ephy_snapshot_service_get_snapshot_for_url:
- * @service: a #EphySnapshotService
- * @url: the URL for which a snapshot is needed
- * @mtime: @the last
- * @callback: a #EphySnapshotServiceCallback
- * @user_data: user data to pass to @callback
- *
- * Schedules a query for a snapshot of @url. If there is an up-to-date
- * snapshot in the cache, this will be retrieved.
- *
- **/
-void
-ephy_snapshot_service_get_snapshot_for_url_async (EphySnapshotService *service,
-                                                  const char *url,
-                                                  const time_t mtime,
-                                                  GCancellable *cancellable,
-                                                  GAsyncReadyCallback callback,
-                                                  gpointer user_data)
-{
-  GTask *task;
-
-  g_return_if_fail (EPHY_IS_SNAPSHOT_SERVICE (service));
-  g_return_if_fail (url != NULL);
-
-  task = g_task_new (service, cancellable, callback, user_data);
-  g_task_set_priority (task, G_PRIORITY_LOW);
-  g_task_set_task_data (task,
-                        snapshot_for_url_async_data_new (url, mtime),
-                        (GDestroyNotify)snapshot_for_url_async_data_free);
-  g_task_run_in_thread (task, (GTaskThreadFunc)get_snapshot_for_url_thread);
-  g_object_unref (task);
-}
-
-/**
- * ephy_snapshot_service_get_snapshot_for_url_finish:
- * @service: a #EphySnapshotService
- * @result: a #GAsyncResult
- * @error: a location to store a #GError or %NULL
- *
- * Finishes the retrieval of a snapshot. Call from the
- * #GAsyncReadyCallback passed to
- * ephy_snapshot_service_get_snapshot_for_url_async().
- *
- * Returns: (transfer full): the snapshot.
- **/
-GdkPixbuf *
-ephy_snapshot_service_get_snapshot_for_url_finish (EphySnapshotService *service,
-                                                   GAsyncResult *result,
-                                                   gchar **path,
-                                                   GError **error)
-{
-  GTask *task = G_TASK (result);
-  GdkPixbuf *snapshot;
-
-  g_return_val_if_fail (g_task_is_valid (result, service), NULL);
-
-  snapshot = g_task_propagate_pointer (task, error);
-  if (!snapshot)
-    return NULL;
-
-  if (path) {
-    SnapshotForURLAsyncData *data;
-
-    data = g_task_get_task_data (task);
-    *path = data->path;
-    data->path = NULL;
-  }
-
-  return snapshot;
-}
-
-static void
-got_snapshot_for_url (EphySnapshotService *service,
-                      GAsyncResult *result,
-                      GTask *task)
-{
-  GdkPixbuf *snapshot;
-  SnapshotAsyncData *data;
-
-  data = g_task_get_task_data (task);
-  snapshot = ephy_snapshot_service_get_snapshot_for_url_finish (service, result, &data->path, NULL);
-  if (snapshot) {
-    g_task_return_pointer (task, snapshot, g_object_unref);
-    g_object_unref (task);
-    return;
-  }
-
-  ephy_snapshot_service_take_from_webview (task);
-}
-
-/* We want to return an existing snapshot immediately, even if it is stale,
- * because snapshot creation is best-effort and often fails (e.g. if the user
- * navigates away from the page too soon), and we must be sure to return an old
- * result if a new one does not yet exist.
- */
-static void
-ensure_snapshot_freshness_for_web_view (EphySnapshotService *service,
-                                        WebKitWebView       *web_view)
-{
-  GTask *task;
-  const char *uri;
-
-  uri = webkit_web_view_get_uri (web_view);
-  if (ephy_snapshot_service_lookup_snapshot_freshness (service, uri) != SNAPSHOT_FRESH) {
-    task = g_task_new (service, NULL, NULL, NULL);
-    g_task_set_task_data (task,
-                          snapshot_async_data_new (web_view, time (NULL)),
-                          (GDestroyNotify)snapshot_async_data_free);
-    g_idle_add ((GSourceFunc)ephy_snapshot_service_take_from_webview, task);
-  }
-}
-
-/**
- * ephy_snapshot_service_get_snapshot_async:
- * @service: a #EphySnapshotService
- * @web_view: the #WebKitWebView for which a snapshot is needed
- * @mtime: @the last
- * @callback: a #EphySnapshotServiceCallback
- * @user_data: user data to pass to @callback
- *
- * Schedules a query for a snapshot of @url. If there is an up-to-date
- * snapshot in the cache, this will be retrieved. Otherwise, this
- * the snapshot will be taken, cached, and retrieved.
- *
- **/
-void
-ephy_snapshot_service_get_snapshot_async (EphySnapshotService *service,
-                                          WebKitWebView *web_view,
-                                          const time_t mtime,
-                                          GCancellable *cancellable,
-                                          GAsyncReadyCallback callback,
-                                          gpointer user_data)
-{
-  GTask *task;
-  const char *uri;
-
-  g_return_if_fail (EPHY_IS_SNAPSHOT_SERVICE (service));
-  g_return_if_fail (WEBKIT_IS_WEB_VIEW (web_view));
-
-  task = g_task_new (service, cancellable, callback, user_data);
-  g_task_set_task_data (task,
-                        snapshot_async_data_new_for_snapshot (web_view, mtime),
-                        (GDestroyNotify)snapshot_async_data_free);
-
-  /* Try to get the snapshot from the cache first if we have a URL, but only if
-   * the snapshot path is in memory cache; this is an indication that the
-   * snapshot is fresh. */
-  uri = webkit_web_view_get_uri (web_view);
-  if (uri) {
-    ephy_snapshot_service_get_snapshot_for_url_async (service,
-                                                      uri, mtime, cancellable,
-                                                      (GAsyncReadyCallback)got_snapshot_for_url,
-                                                      task);
-    ensure_snapshot_freshness_for_web_view (service, web_view);
-  } else {
-    g_idle_add ((GSourceFunc)ephy_snapshot_service_take_from_webview, task);
-  }
-}
-
-/**
- * ephy_snapshot_service_get_snapshot_finish:
- * @service: a #EphySnapshotService
- * @result: a #GAsyncResult
- * @error: a location to store a #GError or %NULL
- *
- * Finishes the retrieval of a snapshot. Call from the
- * #GAsyncReadyCallback passed to
- * ephy_snapshot_service_get_snapshot_async().
- *
- * Returns: (transfer full): the snapshot.
- **/
-GdkPixbuf *
-ephy_snapshot_service_get_snapshot_finish (EphySnapshotService *service,
-                                           GAsyncResult *result,
-                                           gchar **path,
-                                           GError **error)
-{
-  GTask *task = G_TASK (result);
-  GdkPixbuf *snapshot;
-
-  g_return_val_if_fail (g_task_is_valid (result, service), NULL);
-
-  snapshot = g_task_propagate_pointer (task, error);
-  if (!snapshot)
-    return NULL;
-
-  if (path) {
-    SnapshotAsyncData *data;
-
-    data = g_task_get_task_data (task);
-    *path = data->path;
-    data->path = NULL;
-  }
-
-  return snapshot;
-}
-
-typedef struct {
-  EphySnapshotService *service;
-  GdkPixbuf *snapshot;
-  char *url;
-  time_t mtime;
-  gint refcount;
-} SaveSnapshotAsyncData;
-
-static SaveSnapshotAsyncData *
-save_snapshot_async_data_new (EphySnapshotService *service,
-                              GdkPixbuf           *snapshot,
-                              const char          *url,
-                              time_t               mtime)
-{
-  SaveSnapshotAsyncData *data;
-
-  data = g_slice_new0 (SaveSnapshotAsyncData);
-  data->service = g_object_ref (service);
-  data->snapshot = g_object_ref (snapshot);
-  data->url = g_strdup (url);
-  data->mtime = mtime;
-  data->refcount = 1;
-
-  return data;
-}
-
-static SaveSnapshotAsyncData *
-save_snapshot_async_data_ref (SaveSnapshotAsyncData *data)
-{
-  g_atomic_int_add (&data->refcount, 1);
-  return data;
-}
-
-static void
-save_snapshot_async_data_unref (SaveSnapshotAsyncData *data)
-{
-  if (g_atomic_int_dec_and_test (&data->refcount)) {
-    g_object_unref (data->service);
-    g_object_unref (data->snapshot);
-    g_free (data->url);
-    g_slice_free (SaveSnapshotAsyncData, data);
-  }
-}
-
-static gboolean
-idle_emit_snapshot_saved (gpointer user_data)
-{
-  SaveSnapshotAsyncData *data = (SaveSnapshotAsyncData *)user_data;
-
-  g_signal_emit (data->service, signals[SNAPSHOT_SAVED], 0, data->url, data->mtime);
-
-  save_snapshot_async_data_unref (data);
-  return G_SOURCE_REMOVE;
-}
-
-static void
-save_snapshot_thread (GTask *task,
-                      EphySnapshotService *service,
-                      SaveSnapshotAsyncData *data,
-                      GCancellable *cancellable)
-{
-  char *path;
-
-  gnome_desktop_thumbnail_factory_save_thumbnail (service->priv->factory,
-                                                  data->snapshot,
-                                                  data->url,
-                                                  data->mtime);
-  g_idle_add (idle_emit_snapshot_saved, save_snapshot_async_data_ref (data));
-
-  path = gnome_desktop_thumbnail_path_for_uri (data->url, GNOME_DESKTOP_THUMBNAIL_SIZE_LARGE);
-  cache_snapshot_data_in_idle (service, data->url, path, SNAPSHOT_FRESH);
-
-  g_task_return_pointer (task, path, g_free);
-}
-
-void
-ephy_snapshot_service_save_snapshot_async (EphySnapshotService *service,
-                                           GdkPixbuf *snapshot,
-                                           const char *url,
-                                           time_t mtime,
-                                           GCancellable *cancellable,
-                                           GAsyncReadyCallback callback,
-                                           gpointer user_data)
-{
-  GTask *task;
-
-  g_return_if_fail (EPHY_IS_SNAPSHOT_SERVICE (service));
-  g_return_if_fail (GDK_IS_PIXBUF (snapshot));
-  g_return_if_fail (url != NULL);
-
-  task = g_task_new (service, cancellable, callback, user_data);
-  g_task_set_priority (task, G_PRIORITY_LOW);
-  g_task_set_task_data (task,
-                        save_snapshot_async_data_new (service, snapshot, url, mtime),
-                        (GDestroyNotify)save_snapshot_async_data_unref);
-  g_task_run_in_thread (task, (GTaskThreadFunc)save_snapshot_thread);
-  g_object_unref (task);
-}
-
-char *
-ephy_snapshot_service_save_snapshot_finish (EphySnapshotService *service,
-                                            GAsyncResult *result,
-                                            GError **error)
-{
-  g_return_val_if_fail (g_task_is_valid (result, service), NULL);
-
-  return g_task_propagate_pointer (G_TASK (result), error);
-}
-
-static void
-get_snapshot_path_for_url_thread (GTask *task,
+get_snapshot_path_for_url_thread (GTask               *task,
                                   EphySnapshotService *service,
-                                  SnapshotForURLAsyncData *data,
-                                  GCancellable *cancellable)
+                                  SnapshotAsyncData   *data,
+                                  GCancellable        *cancellable)
 {
   char *path;
 
-  path = gnome_desktop_thumbnail_factory_lookup (service->priv->factory, data->url, data->mtime);
+  path = gnome_desktop_thumbnail_factory_lookup (service->factory, data->url, data->mtime);
   if (!path) {
     g_task_return_new_error (task,
                              EPHY_SNAPSHOT_SERVICE_ERROR,
                              EPHY_SNAPSHOT_SERVICE_ERROR_NOT_FOUND,
-                             "Snapshot for url \"%s\" not found in cache", data->url);
+                             "Snapshot for url \"%s\" not found in disk cache", data->url);
     return;
   }
 
@@ -842,11 +536,11 @@ get_snapshot_path_for_url_thread (GTask *task,
 
 void
 ephy_snapshot_service_get_snapshot_path_for_url_async (EphySnapshotService *service,
-                                                       const char *url,
-                                                       const time_t mtime,
-                                                       GCancellable *cancellable,
-                                                       GAsyncReadyCallback callback,
-                                                       gpointer user_data)
+                                                       const char          *url,
+                                                       time_t               mtime,
+                                                       GCancellable        *cancellable,
+                                                       GAsyncReadyCallback  callback,
+                                                       gpointer             user_data)
 {
   GTask *task;
   const char *path;
@@ -856,8 +550,7 @@ ephy_snapshot_service_get_snapshot_path_for_url_async (EphySnapshotService *serv
 
   task = g_task_new (service, cancellable, callback, user_data);
 
-  path = ephy_snapshot_service_lookup_snapshot_path (service, url);
-
+  path = ephy_snapshot_service_lookup_cached_snapshot_path (service, url);
   if (path) {
     g_task_return_pointer (task, g_strdup (path), g_free);
     g_object_unref (task);
@@ -866,16 +559,33 @@ ephy_snapshot_service_get_snapshot_path_for_url_async (EphySnapshotService *serv
 
   g_task_set_priority (task, G_PRIORITY_LOW);
   g_task_set_task_data (task,
-                        snapshot_for_url_async_data_new (url, mtime),
-                        (GDestroyNotify)snapshot_for_url_async_data_free);
+                        snapshot_async_data_new (service, NULL, NULL, mtime, url),
+                        (GDestroyNotify)snapshot_async_data_free);
   g_task_run_in_thread (task, (GTaskThreadFunc)get_snapshot_path_for_url_thread);
   g_object_unref (task);
 }
 
+static void
+take_fresh_snapshot_in_background_if_stale (EphySnapshotService *service,
+                                            SnapshotAsyncData   *data)
+{
+  GTask *task;
+
+  /* We schedule a new snapshot now, which will complete eventually. It won't be
+   * used now. This is just to ensure we get a newer snapshot in the future. */
+  if (ephy_snapshot_service_lookup_snapshot_freshness (service, data->url) == SNAPSHOT_STALE) {
+    task = g_task_new (service, NULL, NULL, NULL);
+    g_task_set_task_data (task,
+                          data,
+                          (GDestroyNotify)snapshot_async_data_free);
+    ephy_snapshot_service_take_from_webview (task);
+  }
+}
+
 char *
 ephy_snapshot_service_get_snapshot_path_for_url_finish (EphySnapshotService *service,
-                                                        GAsyncResult *result,
-                                                        GError **error)
+                                                        GAsyncResult        *result,
+                                                        GError             **error)
 {
   g_return_val_if_fail (g_task_is_valid (result, service), NULL);
 
@@ -884,13 +594,15 @@ ephy_snapshot_service_get_snapshot_path_for_url_finish (EphySnapshotService *ser
 
 static void
 got_snapshot_path_for_url (EphySnapshotService *service,
-                           GAsyncResult *result,
-                           GTask *task)
+                           GAsyncResult        *result,
+                           GTask               *task)
 {
+  SnapshotAsyncData *data = g_task_get_task_data (task);
   char *path;
 
   path = ephy_snapshot_service_get_snapshot_path_for_url_finish (service, result, NULL);
   if (path) {
+    take_fresh_snapshot_in_background_if_stale (service, snapshot_async_data_copy (data));
     g_task_return_pointer (task, path, g_free);
     g_object_unref (task);
   } else {
@@ -900,49 +612,45 @@ got_snapshot_path_for_url (EphySnapshotService *service,
 
 void
 ephy_snapshot_service_get_snapshot_path_async (EphySnapshotService *service,
-                                               WebKitWebView *web_view,
-                                               const time_t mtime,
-                                               GCancellable *cancellable,
-                                               GAsyncReadyCallback callback,
-                                               gpointer user_data)
+                                               WebKitWebView       *web_view,
+                                               time_t               mtime,
+                                               GCancellable        *cancellable,
+                                               GAsyncReadyCallback  callback,
+                                               gpointer             user_data)
 {
   GTask *task;
   const char *uri;
+  const char *path;
 
   g_return_if_fail (EPHY_IS_SNAPSHOT_SERVICE (service));
   g_return_if_fail (WEBKIT_IS_WEB_VIEW (web_view));
+  g_return_if_fail (webkit_web_view_get_uri (web_view));
 
   task = g_task_new (service, cancellable, callback, user_data);
 
   uri = webkit_web_view_get_uri (web_view);
-  if (uri) {
-    const char *path = ephy_snapshot_service_lookup_snapshot_path (service, uri);
-    if (path) {
-      g_task_return_pointer (task, g_strdup (path), g_free);
-      g_object_unref (task);
-    } else {
-      g_task_set_task_data (task,
-                            snapshot_async_data_new (web_view, mtime),
-                            (GDestroyNotify)snapshot_async_data_free);
-      ephy_snapshot_service_get_snapshot_path_for_url_async (service,
-                                                             uri, mtime, cancellable,
-                                                             (GAsyncReadyCallback)got_snapshot_path_for_url,
-                                                             task);
-    }
+  path = ephy_snapshot_service_lookup_cached_snapshot_path (service, uri);
 
-    ensure_snapshot_freshness_for_web_view (service, web_view);
+  if (path) {
+    take_fresh_snapshot_in_background_if_stale (service,
+                                                snapshot_async_data_new (service, NULL, web_view, mtime, uri));
+    g_task_return_pointer (task, g_strdup (path), g_free);
+    g_object_unref (task);
   } else {
     g_task_set_task_data (task,
-                          snapshot_async_data_new (web_view, mtime),
+                          snapshot_async_data_new (service, NULL, web_view, mtime, uri),
                           (GDestroyNotify)snapshot_async_data_free);
-    g_idle_add ((GSourceFunc)ephy_snapshot_service_take_from_webview, task);
+    ephy_snapshot_service_get_snapshot_path_for_url_async (service,
+                                                           uri, mtime, cancellable,
+                                                           (GAsyncReadyCallback)got_snapshot_path_for_url,
+                                                           task);
   }
 }
 
 char *
 ephy_snapshot_service_get_snapshot_path_finish (EphySnapshotService *service,
-                                                GAsyncResult *result,
-                                                GError **error)
+                                                GAsyncResult        *result,
+                                                GError             **error)
 {
   g_return_val_if_fail (g_task_is_valid (result, service), NULL);
 
