@@ -12,23 +12,33 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 #include "ephy-sqlite-connection.h"
 
+#include "ephy-lib-type-builtins.h"
+
 #include <sqlite3.h>
 
-struct _EphySQLiteConnectionPrivate {
+struct _EphySQLiteConnection {
+  GObject parent_instance;
+
   sqlite3 *database;
+  EphySQLiteConnectionMode mode;
 };
 
-#define EPHY_SQLITE_CONNECTION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE((o), EPHY_TYPE_SQLITE_CONNECTION, EphySQLiteConnectionPrivate))
-
 G_DEFINE_TYPE (EphySQLiteConnection, ephy_sqlite_connection, G_TYPE_OBJECT);
 
+enum {
+  PROP_0,
+  PROP_MODE,
+  LAST_PROP
+};
+
+static GParamSpec *obj_properties[LAST_PROP];
+
 static void
 ephy_sqlite_connection_finalize (GObject *self)
 {
@@ -36,52 +46,84 @@ ephy_sqlite_connection_finalize (GObject *self)
   G_OBJECT_CLASS (ephy_sqlite_connection_parent_class)->finalize (self);
 }
 
+static void
+ephy_sqlite_connection_set_property (GObject      *object,
+                                     guint         property_id,
+                                     const GValue *value,
+                                     GParamSpec   *pspec)
+{
+  EphySQLiteConnection *self = EPHY_SQLITE_CONNECTION (object);
+
+  switch (property_id) {
+    case PROP_MODE:
+      self->mode = g_value_get_enum (value);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (self, property_id, pspec);
+      break;
+  }
+}
+
 static void
 ephy_sqlite_connection_class_init (EphySQLiteConnectionClass *klass)
 {
-  GObjectClass *object_class = G_OBJECT_CLASS (klass);
-  object_class->finalize = ephy_sqlite_connection_finalize;
-  g_type_class_add_private (object_class, sizeof (EphySQLiteConnectionPrivate));
+  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
+
+  gobject_class->finalize = ephy_sqlite_connection_finalize;
+  gobject_class->set_property = ephy_sqlite_connection_set_property;
+
+  obj_properties[PROP_MODE] =
+    g_param_spec_enum ("mode",
+                       "SQLite connection mode",
+                       "Whether the SQLite connection is read-only or writable",
+                       EPHY_TYPE_SQ_LITE_CONNECTION_MODE,
+                       EPHY_SQLITE_CONNECTION_MODE_READWRITE,
+                       G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS);
+
+  g_object_class_install_properties (gobject_class, LAST_PROP, obj_properties);
 }
 
 static void
 ephy_sqlite_connection_init (EphySQLiteConnection *self)
 {
-  self->priv = EPHY_SQLITE_CONNECTION_GET_PRIVATE (self);
-  self->priv->database = NULL;
+  self->database = NULL;
 }
 
-static GQuark get_ephy_sqlite_quark (void)
+GQuark ephy_sqlite_error_quark (void)
 {
   return g_quark_from_static_string ("ephy-sqlite");
 }
 
 static void
-set_error_from_string (const char* string, GError **error)
+set_error_from_string (const char *string, GError **error)
 {
   if (error)
-    *error = g_error_new (get_ephy_sqlite_quark (), 0, string, 0);
+    *error = g_error_new_literal (ephy_sqlite_error_quark (), 0, string);
 }
 
 EphySQLiteConnection *
-ephy_sqlite_connection_new ()
+ephy_sqlite_connection_new (EphySQLiteConnectionMode mode)
 {
-  return EPHY_SQLITE_CONNECTION (g_object_new (EPHY_TYPE_SQLITE_CONNECTION, NULL));
+  return EPHY_SQLITE_CONNECTION (g_object_new (EPHY_TYPE_SQLITE_CONNECTION,
+                                               "mode", mode,
+                                               NULL));
 }
 
 gboolean
 ephy_sqlite_connection_open (EphySQLiteConnection *self, const gchar *filename, GError **error)
 {
-  EphySQLiteConnectionPrivate *priv = self->priv;
-
-  if (priv->database) {
+  if (self->database) {
     set_error_from_string ("Connection already open.", error);
     return FALSE;
   }
-  
-  if (sqlite3_open (filename, &priv->database) != SQLITE_OK) {
+
+  if (sqlite3_open_v2 (filename,
+                       &self->database,
+                       self->mode == EPHY_SQLITE_CONNECTION_MODE_READ_ONLY ? SQLITE_OPEN_READONLY
+                                                                           : SQLITE_OPEN_READWRITE | SQLITE_OPEN_CREATE,
+                       NULL) != SQLITE_OK) {
     ephy_sqlite_connection_get_error (self, error);
-    priv->database = NULL;
+    self->database = NULL;
     return FALSE;
   }
 
@@ -91,10 +133,9 @@ ephy_sqlite_connection_open (EphySQLiteConnection *self, const gchar *filename,
 void
 ephy_sqlite_connection_close (EphySQLiteConnection *self)
 {
-  EphySQLiteConnectionPrivate *priv = self->priv;
-  if (priv->database) {
-    sqlite3_close (priv->database);
-    priv->database = NULL;
+  if (self->database) {
+    sqlite3_close (self->database);
+    self->database = NULL;
   }
 }
 
@@ -102,34 +143,35 @@ void
 ephy_sqlite_connection_get_error (EphySQLiteConnection *self, GError **error)
 {
   if (error)
-    *error = g_error_new (get_ephy_sqlite_quark (), 0, sqlite3_errmsg (self->priv->database), 0);
+    *error = g_error_new_literal (ephy_sqlite_error_quark (), sqlite3_errcode (self->database), sqlite3_errmsg (self->database));
 }
 
 gboolean
 ephy_sqlite_connection_execute (EphySQLiteConnection *self, const char *sql, GError **error)
 {
-  EphySQLiteConnectionPrivate *priv = self->priv;
-  
-  if (priv->database == NULL) {
+  if (self->database == NULL) {
     set_error_from_string ("Connection not open.", error);
     return FALSE;
   }
-  
-  return sqlite3_exec (priv->database, sql, NULL, NULL, NULL) == SQLITE_OK;
+
+  if (sqlite3_exec (self->database, sql, NULL, NULL, NULL) != SQLITE_OK) {
+    ephy_sqlite_connection_get_error (self, error);
+    return FALSE;
+  }
+  return TRUE;
 }
 
 EphySQLiteStatement *
 ephy_sqlite_connection_create_statement (EphySQLiteConnection *self, const char *sql, GError **error)
 {
-  EphySQLiteConnectionPrivate *priv = self->priv;
   sqlite3_stmt *prepared_statement;
 
-  if (priv->database == NULL) {
+  if (self->database == NULL) {
     set_error_from_string ("Connection not open.", error);
     return NULL;
   }
 
-  if (sqlite3_prepare_v2 (priv->database, sql, -1, &prepared_statement, NULL) != SQLITE_OK) {
+  if (sqlite3_prepare_v2 (self->database, sql, -1, &prepared_statement, NULL) != SQLITE_OK) {
     ephy_sqlite_connection_get_error (self, error);
     return NULL;
   }
@@ -143,24 +185,30 @@ ephy_sqlite_connection_create_statement (EphySQLiteConnection *self, const char
 gint64
 ephy_sqlite_connection_get_last_insert_id (EphySQLiteConnection *self)
 {
-  return sqlite3_last_insert_rowid (self->priv->database);
+  return sqlite3_last_insert_rowid (self->database);
 }
 
 gboolean
 ephy_sqlite_connection_begin_transaction (EphySQLiteConnection *self, GError **error)
 {
+  if (self->mode == EPHY_SQLITE_CONNECTION_MODE_READ_ONLY)
+    return TRUE;
   return ephy_sqlite_connection_execute (self, "BEGIN TRANSACTION", error);
 }
 
 gboolean
 ephy_sqlite_connection_rollback_transaction (EphySQLiteConnection *self, GError **error)
 {
+  if (self->mode == EPHY_SQLITE_CONNECTION_MODE_READ_ONLY)
+    return TRUE;
   return ephy_sqlite_connection_execute (self, "ROLLBACK", error);
 }
 
 gboolean
 ephy_sqlite_connection_commit_transaction (EphySQLiteConnection *self, GError **error)
 {
+  if (self->mode == EPHY_SQLITE_CONNECTION_MODE_READ_ONLY)
+    return TRUE;
   return ephy_sqlite_connection_execute (self, "COMMIT", error);
 }
 
@@ -171,7 +219,7 @@ ephy_sqlite_connection_table_exists (EphySQLiteConnection *self, const char *tab
   gboolean table_exists = FALSE;
 
   EphySQLiteStatement *statement = ephy_sqlite_connection_create_statement (self,
-    "SELECT COUNT(type) FROM sqlite_master WHERE type='table' and name=?", &error);
+                                                                            "SELECT COUNT(type) FROM sqlite_master WHERE type='table' and name=?", &error);
   if (error) {
     g_warning ("Could not detect table existence: %s", error->message);
     g_error_free (error);
