@@ -12,8 +12,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -22,20 +21,21 @@
 #include "ephy-sqlite-connection.h"
 #include <sqlite3.h>
 
-enum
-{
+enum {
   PROP_0,
   PROP_PREPARED_STATEMENT,
-  PROP_CONNECTION
+  PROP_CONNECTION,
+  LAST_PROP
 };
 
-struct _EphySQLiteStatementPrivate {
+static GParamSpec *obj_properties[LAST_PROP];
+
+struct _EphySQLiteStatement {
+  GObject parent_instance;
   sqlite3_stmt *prepared_statement;
   EphySQLiteConnection *connection;
 };
 
-#define EPHY_SQLITE_STATEMENT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE((o), EPHY_TYPE_SQLITE_STATEMENT, EphySQLiteStatementPrivate))
-
 G_DEFINE_TYPE (EphySQLiteStatement, ephy_sqlite_statement, G_TYPE_OBJECT);
 
 static void
@@ -45,10 +45,10 @@ ephy_sqlite_statement_set_property (GObject *object, guint property_id, const GV
 
   switch (property_id) {
     case PROP_PREPARED_STATEMENT:
-      self->priv->prepared_statement = g_value_get_pointer (value);
+      self->prepared_statement = g_value_get_pointer (value);
       break;
     case PROP_CONNECTION:
-      self->priv->connection = EPHY_SQLITE_CONNECTION (g_object_ref (g_value_get_object (value)));
+      self->connection = EPHY_SQLITE_CONNECTION (g_object_ref (g_value_get_object (value)));
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (self, property_id, pspec);
@@ -57,21 +57,21 @@ ephy_sqlite_statement_set_property (GObject *object, guint property_id, const GV
 }
 
 static void
-ephy_sqlite_statement_finalize (GObject *self)
+ephy_sqlite_statement_finalize (GObject *object)
 {
-  EphySQLiteStatementPrivate *priv = EPHY_SQLITE_STATEMENT (self)->priv;
+  EphySQLiteStatement *self = EPHY_SQLITE_STATEMENT (object);
 
-  if (priv->prepared_statement) {
-    sqlite3_finalize (priv->prepared_statement);
-    priv->prepared_statement = NULL;
+  if (self->prepared_statement) {
+    sqlite3_finalize (self->prepared_statement);
+    self->prepared_statement = NULL;
   }
 
-  if (priv->connection) {
-    g_object_unref (priv->connection);
-    priv->connection = NULL;
+  if (self->connection) {
+    g_object_unref (self->connection);
+    self->connection = NULL;
   }
 
-  G_OBJECT_CLASS (ephy_sqlite_statement_parent_class)->finalize (self);
+  G_OBJECT_CLASS (ephy_sqlite_statement_parent_class)->finalize (object);
 }
 
 static void
@@ -80,37 +80,35 @@ ephy_sqlite_statement_class_init (EphySQLiteStatementClass *klass)
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   gobject_class->finalize = ephy_sqlite_statement_finalize;
   gobject_class->set_property = ephy_sqlite_statement_set_property;
-  g_type_class_add_private (gobject_class, sizeof (EphySQLiteStatementPrivate));
 
-  g_object_class_install_property (gobject_class,
-                                   PROP_PREPARED_STATEMENT,
-                                   g_param_spec_pointer ("prepared-statement",
-                                                        "Prepared statement",
-                                                        "The statement's backing SQLite prepared statement",
-                                                        G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+  obj_properties[PROP_PREPARED_STATEMENT] =
+    g_param_spec_pointer ("prepared-statement",
+                          "Prepared statement",
+                          "The statement's backing SQLite prepared statement",
+                          G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS);
 
-  g_object_class_install_property (gobject_class,
-                                   PROP_CONNECTION,
-                                   g_param_spec_object ("connection",
-                                                        "Connection",
-                                                        "The statement's backing SQLite connection",
-                                                        EPHY_TYPE_SQLITE_CONNECTION,
-                                                        G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+  obj_properties[PROP_CONNECTION] =
+    g_param_spec_object ("connection",
+                         "Connection",
+                         "The statement's backing SQLite connection",
+                         EPHY_TYPE_SQLITE_CONNECTION,
+                         G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS);
+
+  g_object_class_install_properties (gobject_class, LAST_PROP, obj_properties);
 }
 
 static void
 ephy_sqlite_statement_init (EphySQLiteStatement *self)
 {
-  self->priv = EPHY_SQLITE_STATEMENT_GET_PRIVATE (self);
-  self->priv->prepared_statement = NULL;
-  self->priv->connection = NULL;
+  self->prepared_statement = NULL;
+  self->connection = NULL;
 }
 
 gboolean
 ephy_sqlite_statement_bind_null (EphySQLiteStatement *self, int column, GError **error)
 {
-  if (sqlite3_bind_null (self->priv->prepared_statement, column) != SQLITE_OK) {
-    ephy_sqlite_connection_get_error (self->priv->connection, error);
+  if (sqlite3_bind_null (self->prepared_statement, column) != SQLITE_OK) {
+    ephy_sqlite_connection_get_error (self->connection, error);
     return FALSE;
   }
 
@@ -120,8 +118,8 @@ ephy_sqlite_statement_bind_null (EphySQLiteStatement *self, int column, GError *
 gboolean
 ephy_sqlite_statement_bind_boolean (EphySQLiteStatement *self, int column, gboolean value, GError **error)
 {
-  if (sqlite3_bind_int (self->priv->prepared_statement, column + 1, value ? 1 : 0) != SQLITE_OK) {
-    ephy_sqlite_connection_get_error (self->priv->connection, error);
+  if (sqlite3_bind_int (self->prepared_statement, column + 1, value ? 1 : 0) != SQLITE_OK) {
+    ephy_sqlite_connection_get_error (self->connection, error);
     return FALSE;
   }
 
@@ -131,8 +129,8 @@ ephy_sqlite_statement_bind_boolean (EphySQLiteStatement *self, int column, gbool
 gboolean
 ephy_sqlite_statement_bind_int (EphySQLiteStatement *self, int column, int value, GError **error)
 {
-  if (sqlite3_bind_int (self->priv->prepared_statement, column + 1, value) != SQLITE_OK) {
-    ephy_sqlite_connection_get_error (self->priv->connection, error);
+  if (sqlite3_bind_int (self->prepared_statement, column + 1, value) != SQLITE_OK) {
+    ephy_sqlite_connection_get_error (self->connection, error);
     return FALSE;
   }
 
@@ -142,8 +140,8 @@ ephy_sqlite_statement_bind_int (EphySQLiteStatement *self, int column, int value
 gboolean
 ephy_sqlite_statement_bind_double (EphySQLiteStatement *self, int column, double value, GError **error)
 {
-  if (sqlite3_bind_double (self->priv->prepared_statement, column + 1, value) != SQLITE_OK) {
-    ephy_sqlite_connection_get_error (self->priv->connection, error);
+  if (sqlite3_bind_double (self->prepared_statement, column + 1, value) != SQLITE_OK) {
+    ephy_sqlite_connection_get_error (self->connection, error);
     return FALSE;
   }
 
@@ -153,8 +151,8 @@ ephy_sqlite_statement_bind_double (EphySQLiteStatement *self, int column, double
 gboolean
 ephy_sqlite_statement_bind_string (EphySQLiteStatement *self, int column, const char *value, GError **error)
 {
-  if (sqlite3_bind_text (self->priv->prepared_statement, column + 1, value, -1, SQLITE_TRANSIENT) != SQLITE_OK) {
-    ephy_sqlite_connection_get_error (self->priv->connection, error);
+  if (sqlite3_bind_text (self->prepared_statement, column + 1, value, -1, SQLITE_TRANSIENT) != SQLITE_OK) {
+    ephy_sqlite_connection_get_error (self->connection, error);
     return FALSE;
   }
 
@@ -164,8 +162,8 @@ ephy_sqlite_statement_bind_string (EphySQLiteStatement *self, int column, const
 gboolean
 ephy_sqlite_statement_bind_blob (EphySQLiteStatement *self, int column, const void *value, int length, GError **error)
 {
-  if (sqlite3_bind_blob (self->priv->prepared_statement, column + 1, value, length, SQLITE_TRANSIENT) != SQLITE_OK) {
-    ephy_sqlite_connection_get_error (self->priv->connection, error);
+  if (sqlite3_bind_blob (self->prepared_statement, column + 1, value, length, SQLITE_TRANSIENT) != SQLITE_OK) {
+    ephy_sqlite_connection_get_error (self->connection, error);
     return FALSE;
   }
   return TRUE;
@@ -174,9 +172,9 @@ ephy_sqlite_statement_bind_blob (EphySQLiteStatement *self, int column, const vo
 gboolean
 ephy_sqlite_statement_step (EphySQLiteStatement *self, GError **error)
 {
-  int error_code = sqlite3_step (self->priv->prepared_statement);
+  int error_code = sqlite3_step (self->prepared_statement);
   if (error_code != SQLITE_OK && error_code != SQLITE_ROW && error_code != SQLITE_DONE) {
-    ephy_sqlite_connection_get_error (self->priv->connection, error);
+    ephy_sqlite_connection_get_error (self->connection, error);
   }
 
   return error_code == SQLITE_ROW;
@@ -185,19 +183,19 @@ ephy_sqlite_statement_step (EphySQLiteStatement *self, GError **error)
 void
 ephy_sqlite_statement_reset (EphySQLiteStatement *self)
 {
-  sqlite3_reset (self->priv->prepared_statement);
+  sqlite3_reset (self->prepared_statement);
 }
 
 int
 ephy_sqlite_statement_get_column_count (EphySQLiteStatement *self)
 {
-  return sqlite3_column_count (self->priv->prepared_statement);
+  return sqlite3_column_count (self->prepared_statement);
 }
 
 EphySQLiteColumnType
 ephy_sqlite_statement_get_column_type (EphySQLiteStatement *self, int column)
 {
-  int column_type = sqlite3_column_type (self->priv->prepared_statement, column);
+  int column_type = sqlite3_column_type (self->prepared_statement, column);
   switch (column_type) {
     case SQLITE_INTEGER:
       return EPHY_SQLITE_COLUMN_TYPE_INT;
@@ -216,7 +214,7 @@ ephy_sqlite_statement_get_column_type (EphySQLiteStatement *self, int column)
 int
 ephy_sqlite_statement_get_column_size (EphySQLiteStatement *self, int column)
 {
-  return sqlite3_column_bytes (self->priv->prepared_statement, column);
+  return sqlite3_column_bytes (self->prepared_statement, column);
 }
 
 int
@@ -228,25 +226,25 @@ ephy_sqlite_statement_get_column_as_boolean (EphySQLiteStatement *self, int colu
 int
 ephy_sqlite_statement_get_column_as_int (EphySQLiteStatement *self, int column)
 {
-  return sqlite3_column_int (self->priv->prepared_statement, column);
+  return sqlite3_column_int (self->prepared_statement, column);
 }
 
 double
 ephy_sqlite_statement_get_column_as_double (EphySQLiteStatement *self, int column)
 {
-  return sqlite3_column_double (self->priv->prepared_statement, column);
+  return sqlite3_column_double (self->prepared_statement, column);
 }
 
-const char*
+const char *
 ephy_sqlite_statement_get_column_as_string (EphySQLiteStatement *self, int column)
 {
-  return (const char*) sqlite3_column_text (self->priv->prepared_statement, column);
+  return (const char *)sqlite3_column_text (self->prepared_statement, column);
 }
 
-const void*
+const void *
 ephy_sqlite_statement_get_column_as_blob (EphySQLiteStatement *self, int column)
 {
-  return sqlite3_column_blob (self->priv->prepared_statement, column);
+  return sqlite3_column_blob (self->prepared_statement, column);
 }
 
 char *
