@@ -14,8 +14,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -28,42 +27,39 @@
 gboolean
 ephy_history_service_initialize_hosts_table (EphyHistoryService *self)
 {
-  EphyHistoryServicePrivate *priv = EPHY_HISTORY_SERVICE (self)->priv;
   GError *error = NULL;
 
-  if (ephy_sqlite_connection_table_exists (priv->history_database, "hosts")) {
+  if (ephy_sqlite_connection_table_exists (self->history_database, "hosts")) {
     return TRUE;
   }
-  ephy_sqlite_connection_execute (priv->history_database,
-    "CREATE TABLE hosts ("
-    "id INTEGER PRIMARY KEY,"
-    "url LONGVARCAR,"
-    "title LONGVARCAR,"
-    "visit_count INTEGER DEFAULT 0 NOT NULL,"
-    "zoom_level REAL DEFAULT 1.0)", &error);
+  ephy_sqlite_connection_execute (self->history_database,
+                                  "CREATE TABLE hosts ("
+                                  "id INTEGER PRIMARY KEY,"
+                                  "url LONGVARCAR,"
+                                  "title LONGVARCAR,"
+                                  "visit_count INTEGER DEFAULT 0 NOT NULL,"
+                                  "zoom_level REAL DEFAULT 1.0)", &error);
 
   if (error) {
     g_warning ("Could not create hosts table: %s", error->message);
     g_error_free (error);
     return FALSE;
   }
-  ephy_history_service_schedule_commit (self);
   return TRUE;
 }
 
 void
 ephy_history_service_add_host_row (EphyHistoryService *self, EphyHistoryHost *host)
 {
-  EphyHistoryServicePrivate *priv = EPHY_HISTORY_SERVICE (self)->priv;
   EphySQLiteStatement *statement = NULL;
   GError *error = NULL;
 
-  g_assert (priv->history_thread == g_thread_self ());
-  g_assert (priv->history_database != NULL);
+  g_assert (self->history_thread == g_thread_self ());
+  g_assert (self->history_database != NULL);
 
-  statement = ephy_sqlite_connection_create_statement (priv->history_database,
-    "INSERT INTO hosts (url, title, visit_count, zoom_level) "
-    "VALUES (?, ?, ?, ?)", &error);
+  statement = ephy_sqlite_connection_create_statement (self->history_database,
+                                                       "INSERT INTO hosts (url, title, visit_count, zoom_level) "
+                                                       "VALUES (?, ?, ?, ?)", &error);
 
   if (error) {
     g_warning ("Could not build hosts table addition statement: %s", error->message);
@@ -86,7 +82,7 @@ ephy_history_service_add_host_row (EphyHistoryService *self, EphyHistoryHost *ho
     g_warning ("Could not insert host into hosts table: %s", error->message);
     g_error_free (error);
   } else {
-    host->id = ephy_sqlite_connection_get_last_insert_id (priv->history_database);
+    host->id = ephy_sqlite_connection_get_last_insert_id (self->history_database);
   }
 
   g_object_unref (statement);
@@ -95,16 +91,15 @@ ephy_history_service_add_host_row (EphyHistoryService *self, EphyHistoryHost *ho
 void
 ephy_history_service_update_host_row (EphyHistoryService *self, EphyHistoryHost *host)
 {
-  EphyHistoryServicePrivate *priv = EPHY_HISTORY_SERVICE (self)->priv;
   EphySQLiteStatement *statement;
   GError *error = NULL;
 
-  g_assert (priv->history_thread == g_thread_self ());
-  g_assert (priv->history_database != NULL);
+  g_assert (self->history_thread == g_thread_self ());
+  g_assert (self->history_database != NULL);
 
-  statement = ephy_sqlite_connection_create_statement (priv->history_database,
-    "UPDATE hosts SET url=?, title=?, visit_count=?, zoom_level=?"
-    "WHERE id=?", &error);
+  statement = ephy_sqlite_connection_create_statement (self->history_database,
+                                                       "UPDATE hosts SET url=?, title=?, visit_count=?, zoom_level=?"
+                                                       "WHERE id=?", &error);
   if (error) {
     g_warning ("Could not build hosts table modification statement: %s", error->message);
     g_error_free (error);
@@ -130,29 +125,28 @@ ephy_history_service_update_host_row (EphyHistoryService *self, EphyHistoryHost
   g_object_unref (statement);
 }
 
-EphyHistoryHost*
+EphyHistoryHost *
 ephy_history_service_get_host_row (EphyHistoryService *self, const gchar *host_string, EphyHistoryHost *host)
 {
-  EphyHistoryServicePrivate *priv = EPHY_HISTORY_SERVICE (self)->priv;
   EphySQLiteStatement *statement = NULL;
   GError *error = NULL;
 
-  g_assert (priv->history_thread == g_thread_self ());
-  g_assert (priv->history_database != NULL);
+  g_assert (self->history_thread == g_thread_self ());
+  g_assert (self->history_database != NULL);
 
   if (host_string == NULL && host != NULL)
     host_string = host->url;
 
-  g_assert (host_string || host->id !=-1);
+  g_assert (host_string || host->id != -1);
 
   if (host != NULL && host->id != -1) {
-    statement = ephy_sqlite_connection_create_statement (priv->history_database,
-        "SELECT id, url, title, visit_count, zoom_level FROM hosts "
-        "WHERE id=?", &error);
+    statement = ephy_sqlite_connection_create_statement (self->history_database,
+                                                         "SELECT id, url, title, visit_count, zoom_level FROM hosts "
+                                                         "WHERE id=?", &error);
   } else {
-    statement = ephy_sqlite_connection_create_statement (priv->history_database,
-        "SELECT id, url, title, visit_count, zoom_level FROM hosts "
-        "WHERE url=?", &error);
+    statement = ephy_sqlite_connection_create_statement (self->history_database,
+                                                         "SELECT id, url, title, visit_count, zoom_level FROM hosts "
+                                                         "WHERE url=?", &error);
   }
 
   if (error) {
@@ -199,7 +193,7 @@ ephy_history_service_get_host_row (EphyHistoryService *self, const gchar *host_s
   return host;
 }
 
-static EphyHistoryHost*
+static EphyHistoryHost *
 create_host_from_statement (EphySQLiteStatement *statement)
 {
   EphyHistoryHost *host =
@@ -212,19 +206,18 @@ create_host_from_statement (EphySQLiteStatement *statement)
   return host;
 }
 
-GList*
+GList *
 ephy_history_service_get_all_hosts (EphyHistoryService *self)
 {
-  EphyHistoryServicePrivate *priv = EPHY_HISTORY_SERVICE (self)->priv;
   EphySQLiteStatement *statement = NULL;
   GList *hosts = NULL;
   GError *error = NULL;
 
-  g_assert (priv->history_thread == g_thread_self ());
-  g_assert (priv->history_database != NULL);
+  g_assert (self->history_thread == g_thread_self ());
+  g_assert (self->history_database != NULL);
 
-  statement = ephy_sqlite_connection_create_statement (priv->history_database,
-      "SELECT id, url, title, visit_count, zoom_level FROM hosts", &error);
+  statement = ephy_sqlite_connection_create_statement (self->history_database,
+                                                       "SELECT id, url, title, visit_count, zoom_level FROM hosts", &error);
 
   if (error) {
     g_warning ("Could not build hosts query statement: %s", error->message);
@@ -246,35 +239,34 @@ ephy_history_service_get_all_hosts (EphyHistoryService *self)
   return hosts;
 }
 
-GList*
+GList *
 ephy_history_service_find_host_rows (EphyHistoryService *self, EphyHistoryQuery *query)
 {
-  EphyHistoryServicePrivate *priv = EPHY_HISTORY_SERVICE (self)->priv;
   EphySQLiteStatement *statement = NULL;
   GList *substring;
   GString *statement_str;
   GList *hosts = NULL;
   GError *error = NULL;
   const char *base_statement = ""
-    "SELECT "
-      "DISTINCT hosts.id, "
-      "hosts.url, "
-      "hosts.title, "
-      "hosts.visit_count, "
-      "hosts.zoom_level "
-    "FROM "
-      "hosts ";
+                               "SELECT "
+                               "DISTINCT hosts.id, "
+                               "hosts.url, "
+                               "hosts.title, "
+                               "hosts.visit_count, "
+                               "hosts.zoom_level "
+                               "FROM "
+                               "hosts ";
 
   int i = 0;
 
-  g_assert (priv->history_thread == g_thread_self ());
-  g_assert (priv->history_database != NULL);
+  g_assert (self->history_thread == g_thread_self ());
+  g_assert (self->history_database != NULL);
 
   statement_str = g_string_new (base_statement);
 
   /* In either of these cases we need to at least join with the urls table. */
   if (query->substring_list || query->from > 0 || query->to > 0)
-    statement_str = g_string_append (statement_str,  "JOIN urls on hosts.id = urls.host ");
+    statement_str = g_string_append (statement_str, "JOIN urls on hosts.id = urls.host ");
 
   /* In these cases, we additionally need to join with the visits table. */
   if (query->from > 0 || query->to > 0) {
@@ -293,8 +285,8 @@ ephy_history_service_find_host_rows (EphyHistoryService *self, EphyHistoryQuery
 
   statement_str = g_string_append (statement_str, "1 ");
 
-  statement = ephy_sqlite_connection_create_statement (priv->history_database,
-						       statement_str->str, &error);
+  statement = ephy_sqlite_connection_create_statement (self->history_database,
+                                                       statement_str->str, &error);
   g_string_free (statement_str, TRUE);
 
   if (error) {
@@ -323,7 +315,7 @@ ephy_history_service_find_host_rows (EphyHistoryService *self, EphyHistoryQuery
     char *string = ephy_sqlite_create_match_pattern (substring->data);
     while (j--)
       /* The bitwise operation ensures we only skip two characters for titles. */
-      if (ephy_sqlite_statement_bind_string (statement, i++, string + 2*((j+1) & 1), &error) == FALSE) {
+      if (ephy_sqlite_statement_bind_string (statement, i++, string + 2 * ((j + 1) & 1), &error) == FALSE) {
         g_warning ("Could not build hosts table query statement: %s", error->message);
         g_error_free (error);
         g_object_unref (statement);
@@ -351,13 +343,13 @@ ephy_history_service_find_host_rows (EphyHistoryService *self, EphyHistoryQuery
 static GList *
 get_hostname_and_locations (const gchar *url, gchar **hostname)
 {
-	GList *host_locations = NULL;
-	char *scheme = NULL;
+  GList *host_locations = NULL;
+  char *scheme = NULL;
 
   if (url) {
-		scheme = g_uri_parse_scheme (url);
-		*hostname = ephy_string_get_host_name (url);
-	}
+    scheme = g_uri_parse_scheme (url);
+    *hostname = ephy_string_get_host_name (url);
+  }
   /* Build an host name */
   if (scheme == NULL || *hostname == NULL) {
     *hostname = g_strdup (_("Others"));
@@ -398,9 +390,9 @@ get_hostname_and_locations (const gchar *url, gchar **hostname)
   return host_locations;
 }
 
-EphyHistoryHost*
+EphyHistoryHost *
 ephy_history_service_get_host_row_from_url (EphyHistoryService *self,
-                                            const gchar *url)
+                                            const gchar        *url)
 {
   GList *host_locations, *l;
   char *hostname;
@@ -416,37 +408,36 @@ ephy_history_service_get_host_row_from_url (EphyHistoryService *self,
 
   if (host == NULL) {
     host = ephy_history_host_new (host_locations->data, hostname, 0, 1.0);
-    ephy_history_service_add_host_row (self, host);
+    if (!self->read_only)
+      ephy_history_service_add_host_row (self, host);
   }
 
   g_free (hostname);
-  g_list_free_full (host_locations, (GDestroyNotify) g_free);
+  g_list_free_full (host_locations, (GDestroyNotify)g_free);
 
   return host;
 }
 
 void
 ephy_history_service_delete_host_row (EphyHistoryService *self,
-                                      EphyHistoryHost *host)
+                                      EphyHistoryHost    *host)
 {
-  EphyHistoryServicePrivate *priv = EPHY_HISTORY_SERVICE (self)->priv;
   EphySQLiteStatement *statement = NULL;
-  gchar *sql_statement;
+  const char *sql_statement;
   GError *error = NULL;
 
-  g_assert (priv->history_thread == g_thread_self ());
-  g_assert (priv->history_database != NULL);
+  g_assert (self->history_thread == g_thread_self ());
+  g_assert (self->history_database != NULL);
 
   g_assert (host->id != -1 || host->url);
 
   if (host->id != -1)
-    sql_statement = g_strdup ("DELETE FROM hosts WHERE id=?");
+    sql_statement = "DELETE FROM hosts WHERE id=?";
   else
-    sql_statement = g_strdup ("DELETE FROM hosts WHERE url=?");
+    sql_statement = "DELETE FROM hosts WHERE url=?";
 
-  statement = ephy_sqlite_connection_create_statement (priv->history_database,
+  statement = ephy_sqlite_connection_create_statement (self->history_database,
                                                        sql_statement, &error);
-  g_free (sql_statement);
 
   if (error) {
     g_warning ("Could not build urls table query statement: %s", error->message);
@@ -477,18 +468,17 @@ ephy_history_service_delete_host_row (EphyHistoryService *self,
 void
 ephy_history_service_delete_orphan_hosts (EphyHistoryService *self)
 {
-  EphyHistoryServicePrivate *priv = EPHY_HISTORY_SERVICE (self)->priv;
   GError *error = NULL;
 
-  g_assert (priv->history_thread == g_thread_self ());
-  g_assert (priv->history_database != NULL);
+  g_assert (self->history_thread == g_thread_self ());
+  g_assert (self->history_database != NULL);
 
   /* Where a JOIN would give us all hosts with urls associated, a LEFT
      JOIN also gives us those hosts for which there are no urls.  By
      means of urls.host == NULL we filter out anything else and
      retrieve only the ids of the hosts without associated urls. Then,
      we delete all these rows from the hosts table. */
-  ephy_sqlite_connection_execute (priv->history_database,
+  ephy_sqlite_connection_execute (self->history_database,
                                   "DELETE FROM hosts WHERE hosts.id IN "
                                   "  (SELECT hosts.id FROM hosts LEFT JOIN urls "
                                   "    ON hosts.id = urls.host WHERE urls.host is NULL);",
