@@ -13,8 +13,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef EPHY_HISTORY_SERVICE_PRIVATE_H
@@ -22,18 +21,20 @@
 
 #include "ephy-sqlite-connection.h"
 
-struct _EphyHistoryServicePrivate {
+struct _EphyHistoryService {
+  GObject parent_instance;
   char *history_filename;
   EphySQLiteConnection *history_database;
+  GMutex history_thread_mutex;
+  gboolean history_thread_initialized;
+  GCond history_thread_initialized_condition;
   GThread *history_thread;
   GAsyncQueue *queue;
   gboolean scheduled_to_quit;
-  gboolean scheduled_to_commit;
   gboolean read_only;
   int queue_urls_visited_id;
 };
 
-void                     ephy_history_service_schedule_commit         (EphyHistoryService *self); 
 gboolean                 ephy_history_service_initialize_urls_table   (EphyHistoryService *self);
 EphyHistoryURL *         ephy_history_service_get_url_row             (EphyHistoryService *self, const char *url_string, EphyHistoryURL *url);
 void                     ephy_history_service_add_url_row             (EphyHistoryService *self, EphyHistoryURL *url);
