@@ -14,8 +14,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -26,42 +25,39 @@
 gboolean
 ephy_history_service_initialize_urls_table (EphyHistoryService *self)
 {
-  EphyHistoryServicePrivate *priv = EPHY_HISTORY_SERVICE (self)->priv;
   GError *error = NULL;
 
-  if (ephy_sqlite_connection_table_exists (priv->history_database, "visits")) {
+  if (ephy_sqlite_connection_table_exists (self->history_database, "visits")) {
     return TRUE;
   }
-  ephy_sqlite_connection_execute (priv->history_database,
-    "CREATE TABLE urls ("
-    "id INTEGER PRIMARY KEY,"
-    "host INTEGER NOT NULL REFERENCES hosts(id) ON DELETE CASCADE,"
-    "url LONGVARCAR,"
-    "title LONGVARCAR,"
-    "visit_count INTEGER DEFAULT 0 NOT NULL,"
-    "typed_count INTEGER DEFAULT 0 NOT NULL,"
-    "last_visit_time INTEGER,"
-    "thumbnail_update_time INTEGER DEFAULT 0,"
-    "hidden_from_overview INTEGER DEFAULT 0)", &error);
+  ephy_sqlite_connection_execute (self->history_database,
+                                  "CREATE TABLE urls ("
+                                  "id INTEGER PRIMARY KEY,"
+                                  "host INTEGER NOT NULL REFERENCES hosts(id) ON DELETE CASCADE,"
+                                  "url LONGVARCAR,"
+                                  "title LONGVARCAR,"
+                                  "visit_count INTEGER DEFAULT 0 NOT NULL,"
+                                  "typed_count INTEGER DEFAULT 0 NOT NULL,"
+                                  "last_visit_time INTEGER,"
+                                  "thumbnail_update_time INTEGER DEFAULT 0,"
+                                  "hidden_from_overview INTEGER DEFAULT 0)", &error);
 
   if (error) {
     g_warning ("Could not create urls table: %s", error->message);
     g_error_free (error);
     return FALSE;
   }
-  ephy_history_service_schedule_commit (self);
   return TRUE;
 }
 
 EphyHistoryURL *
 ephy_history_service_get_url_row (EphyHistoryService *self, const char *url_string, EphyHistoryURL *url)
 {
-  EphyHistoryServicePrivate *priv = EPHY_HISTORY_SERVICE (self)->priv;
-  EphySQLiteStatement *statement = NULL;  
+  EphySQLiteStatement *statement = NULL;
   GError *error = NULL;
 
-  g_assert (priv->history_thread == g_thread_self ());
-  g_assert (priv->history_database != NULL);
+  g_assert (self->history_thread == g_thread_self ());
+  g_assert (self->history_database != NULL);
 
   if (url_string == NULL && url != NULL)
     url_string = url->url;
@@ -69,13 +65,13 @@ ephy_history_service_get_url_row (EphyHistoryService *self, const char *url_stri
   g_return_val_if_fail (url_string || url->id != -1, NULL);
 
   if (url != NULL && url->id != -1) {
-    statement = ephy_sqlite_connection_create_statement (priv->history_database,
-      "SELECT id, url, title, visit_count, typed_count, last_visit_time, hidden_from_overview, thumbnail_update_time FROM urls "
-      "WHERE id=?", &error);
+    statement = ephy_sqlite_connection_create_statement (self->history_database,
+                                                         "SELECT id, url, title, visit_count, typed_count, last_visit_time, hidden_from_overview, thumbnail_update_time FROM urls "
+                                                         "WHERE id=?", &error);
   } else {
-    statement = ephy_sqlite_connection_create_statement (priv->history_database,
-      "SELECT id, url, title, visit_count, typed_count, last_visit_time, hidden_from_overview, thumbnail_update_time FROM urls "
-      "WHERE url=?", &error);
+    statement = ephy_sqlite_connection_create_statement (self->history_database,
+                                                         "SELECT id, url, title, visit_count, typed_count, last_visit_time, hidden_from_overview, thumbnail_update_time FROM urls "
+                                                         "WHERE url=?", &error);
   }
 
   if (error) {
@@ -127,16 +123,15 @@ ephy_history_service_get_url_row (EphyHistoryService *self, const char *url_stri
 void
 ephy_history_service_add_url_row (EphyHistoryService *self, EphyHistoryURL *url)
 {
-  EphyHistoryServicePrivate *priv = EPHY_HISTORY_SERVICE (self)->priv;
-  EphySQLiteStatement *statement = NULL;  
+  EphySQLiteStatement *statement = NULL;
   GError *error = NULL;
 
-  g_assert (priv->history_thread == g_thread_self ());
-  g_assert (priv->history_database != NULL);
+  g_assert (self->history_thread == g_thread_self ());
+  g_assert (self->history_database != NULL);
 
-  statement = ephy_sqlite_connection_create_statement (priv->history_database,
-    "INSERT INTO urls (url, title, visit_count, typed_count, last_visit_time, host) "
-    " VALUES (?, ?, ?, ?, ?, ?)", &error);
+  statement = ephy_sqlite_connection_create_statement (self->history_database,
+                                                       "INSERT INTO urls (url, title, visit_count, typed_count, last_visit_time, host) "
+                                                       " VALUES (?, ?, ?, ?, ?, ?)", &error);
   if (error) {
     g_warning ("Could not build urls table addition statement: %s", error->message);
     g_error_free (error);
@@ -144,7 +139,7 @@ ephy_history_service_add_url_row (EphyHistoryService *self, EphyHistoryURL *url)
   }
 
   if (ephy_sqlite_statement_bind_string (statement, 0, url->url, &error) == FALSE ||
-      ephy_sqlite_statement_bind_string (statement, 1, url->title, &error) == FALSE || 
+      ephy_sqlite_statement_bind_string (statement, 1, url->title, &error) == FALSE ||
       ephy_sqlite_statement_bind_int (statement, 2, url->visit_count, &error) == FALSE ||
       ephy_sqlite_statement_bind_int (statement, 3, url->typed_count, &error) == FALSE ||
       ephy_sqlite_statement_bind_int (statement, 4, url->last_visit_time, &error) == FALSE ||
@@ -160,7 +155,7 @@ ephy_history_service_add_url_row (EphyHistoryService *self, EphyHistoryURL *url)
     g_warning ("Could not insert URL into urls table: %s", error->message);
     g_error_free (error);
   } else {
-    url->id = ephy_sqlite_connection_get_last_insert_id (priv->history_database);
+    url->id = ephy_sqlite_connection_get_last_insert_id (self->history_database);
   }
 
   g_object_unref (statement);
@@ -169,23 +164,22 @@ ephy_history_service_add_url_row (EphyHistoryService *self, EphyHistoryURL *url)
 void
 ephy_history_service_update_url_row (EphyHistoryService *self, EphyHistoryURL *url)
 {
-  EphyHistoryServicePrivate *priv = EPHY_HISTORY_SERVICE (self)->priv;
   EphySQLiteStatement *statement;
   GError *error = NULL;
 
-  g_assert (priv->history_thread == g_thread_self ());
-  g_assert (priv->history_database != NULL);
+  g_assert (self->history_thread == g_thread_self ());
+  g_assert (self->history_database != NULL);
 
-  statement = ephy_sqlite_connection_create_statement (priv->history_database,
-    "UPDATE urls SET title=?, visit_count=?, typed_count=?, last_visit_time=?, hidden_from_overview=?, thumbnail_update_time=? "
-    "WHERE id=?", &error);
+  statement = ephy_sqlite_connection_create_statement (self->history_database,
+                                                       "UPDATE urls SET title=?, visit_count=?, typed_count=?, last_visit_time=?, hidden_from_overview=?, thumbnail_update_time=? "
+                                                       "WHERE id=?", &error);
   if (error) {
     g_warning ("Could not build urls table modification statement: %s", error->message);
     g_error_free (error);
     return;
   }
 
-  if (ephy_sqlite_statement_bind_string (statement, 0, url->title, &error) == FALSE || 
+  if (ephy_sqlite_statement_bind_string (statement, 0, url->title, &error) == FALSE ||
       ephy_sqlite_statement_bind_int (statement, 1, url->visit_count, &error) == FALSE ||
       ephy_sqlite_statement_bind_int (statement, 2, url->typed_count, &error) == FALSE ||
       ephy_sqlite_statement_bind_int (statement, 3, url->last_visit_time, &error) == FALSE ||
@@ -227,30 +221,29 @@ create_url_from_statement (EphySQLiteStatement *statement)
 GList *
 ephy_history_service_find_url_rows (EphyHistoryService *self, EphyHistoryQuery *query)
 {
-  EphyHistoryServicePrivate *priv = EPHY_HISTORY_SERVICE (self)->priv;
   EphySQLiteStatement *statement = NULL;
   GList *substring;
   GString *statement_str;
   GList *urls = NULL;
   GError *error = NULL;
   const char *base_statement = ""
-    "SELECT "
-      "DISTINCT urls.id, "
-      "urls.url, "
-      "urls.title, "
-      "urls.visit_count, "
-      "urls.typed_count, "
-      "urls.last_visit_time, "
-      "urls.hidden_from_overview, "
-      "urls.thumbnail_update_time, "
-      "urls.host "
-    "FROM "
-      "urls ";
+                               "SELECT "
+                               "DISTINCT urls.id, "
+                               "urls.url, "
+                               "urls.title, "
+                               "urls.visit_count, "
+                               "urls.typed_count, "
+                               "urls.last_visit_time, "
+                               "urls.hidden_from_overview, "
+                               "urls.thumbnail_update_time, "
+                               "urls.host "
+                               "FROM "
+                               "urls ";
 
   int i = 0;
 
-  g_assert (priv->history_thread == g_thread_self ());
-  g_assert (priv->history_database != NULL);
+  g_assert (self->history_thread == g_thread_self ());
+  g_assert (self->history_database != NULL);
 
   statement_str = g_string_new (base_statement);
 
@@ -279,40 +272,41 @@ ephy_history_service_find_url_rows (EphyHistoryService *self, EphyHistoryQuery *
   statement_str = g_string_append (statement_str, "1 ");
 
   switch (query->sort_type) {
-  case EPHY_HISTORY_SORT_MOST_VISITED:
-    statement_str = g_string_append (statement_str, "ORDER BY urls.visit_count DESC ");
-    break;
-  case EPHY_HISTORY_SORT_LEAST_VISITED:
-    statement_str = g_string_append (statement_str, "ORDER BY urls.visit_count ");
-    break;
-  case EPHY_HISTORY_SORT_MOST_RECENTLY_VISITED:
-    statement_str = g_string_append (statement_str, "ORDER BY urls.last_visit_time DESC ");
-    break;
-  case EPHY_HISTORY_SORT_LEAST_RECENTLY_VISITED:
-    statement_str = g_string_append (statement_str, "ORDER BY urls.last_visit_time ");
-    break;
-  case EPHY_HISTORY_SORT_TITLE_ASCENDING:
-    statement_str = g_string_append (statement_str, "ORDER BY LOWER(urls.title) ");
-    break;
-  case EPHY_HISTORY_SORT_TITLE_DESCENDING:
-    statement_str = g_string_append (statement_str, "ORDER BY LOWER(urls.title) DESC ");
-    break;
-  case EPHY_HISTORY_SORT_URL_ASCENDING:
-    statement_str = g_string_append (statement_str, "ORDER BY LOWER(urls.url) ");
-    break;
-  case EPHY_HISTORY_SORT_URL_DESCENDING:
-    statement_str = g_string_append (statement_str, "ORDER BY LOWER(urls.url) DESC ");
-    break;
-  default:
-    g_warning ("We don't support this sorting method yet.");
+    case EPHY_HISTORY_SORT_MOST_VISITED:
+      statement_str = g_string_append (statement_str, "ORDER BY urls.visit_count DESC ");
+      break;
+    case EPHY_HISTORY_SORT_LEAST_VISITED:
+      statement_str = g_string_append (statement_str, "ORDER BY urls.visit_count ");
+      break;
+    case EPHY_HISTORY_SORT_MOST_RECENTLY_VISITED:
+      statement_str = g_string_append (statement_str, "ORDER BY urls.last_visit_time DESC ");
+      break;
+    case EPHY_HISTORY_SORT_LEAST_RECENTLY_VISITED:
+      statement_str = g_string_append (statement_str, "ORDER BY urls.last_visit_time ");
+      break;
+    case EPHY_HISTORY_SORT_TITLE_ASCENDING:
+      statement_str = g_string_append (statement_str, "ORDER BY LOWER(urls.title) ");
+      break;
+    case EPHY_HISTORY_SORT_TITLE_DESCENDING:
+      statement_str = g_string_append (statement_str, "ORDER BY LOWER(urls.title) DESC ");
+      break;
+    case EPHY_HISTORY_SORT_URL_ASCENDING:
+      statement_str = g_string_append (statement_str, "ORDER BY LOWER(urls.url) ");
+      break;
+    case EPHY_HISTORY_SORT_URL_DESCENDING:
+      statement_str = g_string_append (statement_str, "ORDER BY LOWER(urls.url) DESC ");
+      break;
+    case EPHY_HISTORY_SORT_NONE:
+    default:
+      g_warning ("We don't support this sorting method yet.");
   }
 
   if (query->limit) {
     statement_str = g_string_append (statement_str, "LIMIT ? ");
   }
 
-  statement = ephy_sqlite_connection_create_statement (priv->history_database,
-						       statement_str->str, &error);
+  statement = ephy_sqlite_connection_create_statement (self->history_database,
+                                                       statement_str->str, &error);
   g_string_free (statement_str, TRUE);
 
   if (error) {
@@ -392,24 +386,22 @@ ephy_history_service_find_url_rows (EphyHistoryService *self, EphyHistoryQuery *
 void
 ephy_history_service_delete_url (EphyHistoryService *self, EphyHistoryURL *url)
 {
-  EphyHistoryServicePrivate *priv = EPHY_HISTORY_SERVICE (self)->priv;
   EphySQLiteStatement *statement = NULL;
-  gchar *sql_statement;
+  const char *sql_statement;
   GError *error = NULL;
 
-  g_assert (priv->history_thread == g_thread_self ());
-  g_assert (priv->history_database != NULL);
+  g_assert (self->history_thread == g_thread_self ());
+  g_assert (self->history_database != NULL);
 
   g_return_if_fail (url->id != -1 || url->url);
 
   if (url->id != -1)
-    sql_statement = g_strdup ("DELETE FROM urls WHERE id=?");
+    sql_statement = "DELETE FROM urls WHERE id=?";
   else
-    sql_statement = g_strdup ("DELETE FROM urls WHERE url=?");
+    sql_statement = "DELETE FROM urls WHERE url=?";
 
-  statement = ephy_sqlite_connection_create_statement (priv->history_database,
+  statement = ephy_sqlite_connection_create_statement (self->history_database,
                                                        sql_statement, &error);
-  g_free (sql_statement);
 
   if (error) {
     g_warning ("Could not build urls table query statement: %s", error->message);
