@@ -14,15 +14,9 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#if !defined (__EPHY_EPIPHANY_H_INSIDE__) && !defined (EPIPHANY_COMPILATION)
-#error "Only <epiphany/epiphany.h> can be included directly."
-#endif
-
 #ifndef EPHY_LOCATION_ENTRY_H
 #define EPHY_LOCATION_ENTRY_H
 
@@ -32,38 +26,9 @@
 
 G_BEGIN_DECLS
 
-#define EPHY_TYPE_LOCATION_ENTRY		(ephy_location_entry_get_type())
-#define EPHY_LOCATION_ENTRY(object)		(G_TYPE_CHECK_INSTANCE_CAST((object), EPHY_TYPE_LOCATION_ENTRY, EphyLocationEntry))
-#define EPHY_LOCATION_ENTRY_CLASS(klass)	(G_TYPE_CHECK_CLASS_CAST((klass), EPHY_TYPE_LOCATION_ENTRY, EphyLocationEntryClass))
-#define EPHY_IS_LOCATION_ENTRY(object)		(G_TYPE_CHECK_INSTANCE_TYPE((object), EPHY_TYPE_LOCATION_ENTRY))
-#define EPHY_IS_LOCATION_ENTRY_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE((klass), EPHY_TYPE_LOCATION_ENTRY))
-#define EPHY_LOCATION_ENTRY_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS((obj), EPHY_TYPE_LOCATION_ENTRY, EphyLocationEntryClass))
+#define EPHY_TYPE_LOCATION_ENTRY (ephy_location_entry_get_type())
 
-typedef struct _EphyLocationEntryClass		EphyLocationEntryClass;
-typedef struct _EphyLocationEntry		EphyLocationEntry;
-typedef struct _EphyLocationEntryPrivate	EphyLocationEntryPrivate;
-
-struct _EphyLocationEntryClass
-{
-	GtkEntryClass parent_class;
-
-	/* Signals */
-	void   (* user_changed)	(EphyLocationEntry *entry);
-	void   (* lock_clicked)	(EphyLocationEntry *entry);
-	/* for getting the drag data */
-	char * (* get_location)	(EphyLocationEntry *entry);
-	char * (* get_title)	(EphyLocationEntry *entry);
-};
-
-struct _EphyLocationEntry
-{
-	GtkEntry parent_object;
-
-	/*< private >*/
-	EphyLocationEntryPrivate *priv;
-};
-
-GType		ephy_location_entry_get_type		(void);
+G_DECLARE_FINAL_TYPE (EphyLocationEntry, ephy_location_entry, EPHY, LOCATION_ENTRY, GtkEntry)
 
 GtkWidget      *ephy_location_entry_new			(void);
 
