@@ -1,5 +1,5 @@
 /* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* 
+/*
  *  Copyright © 2012 Igalia S.L.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -13,9 +13,7 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -34,24 +32,24 @@ enum {
   PROP_0,
   PROP_HISTORY_SERVICE,
   PROP_BOOKMARKS,
-  PROP_USE_MARKUP
+  LAST_PROP
 };
 
-G_DEFINE_TYPE (EphyCompletionModel, ephy_completion_model, GTK_TYPE_LIST_STORE)
+struct _EphyCompletionModel {
+  GtkListStore parent_instance;
 
-#define EPHY_COMPLETION_MODEL_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_COMPLETION_MODEL, EphyCompletionModelPrivate))
-
-struct _EphyCompletionModelPrivate {
   EphyHistoryService *history_service;
   GCancellable *cancellable;
 
   EphyNode *bookmarks;
   EphyNode *smart_bookmarks;
   GSList *search_terms;
-
-  gboolean use_markup;
 };
 
+static GParamSpec *obj_properties[LAST_PROP];
+
+G_DEFINE_TYPE (EphyCompletionModel, ephy_completion_model, GTK_TYPE_LIST_STORE)
+
 static void
 ephy_completion_model_constructed (GObject *object)
 {
@@ -59,6 +57,8 @@ ephy_completion_model_constructed (GObject *object)
                          G_TYPE_INT, G_TYPE_STRING, G_TYPE_BOOLEAN,
                          GDK_TYPE_PIXBUF };
 
+  G_OBJECT_CLASS (ephy_completion_model_parent_class)->constructed (object);
+
   gtk_list_store_set_column_types (GTK_LIST_STORE (object),
                                    N_COL,
                                    types);
@@ -68,10 +68,10 @@ static void
 free_search_terms (GSList *search_terms)
 {
   GSList *iter;
-  
+
   for (iter = search_terms; iter != NULL; iter = iter->next)
-    g_regex_unref ((GRegex*)iter->data);
-  
+    g_regex_unref ((GRegex *)iter->data);
+
   g_slist_free (search_terms);
 }
 
@@ -81,38 +81,35 @@ ephy_completion_model_set_property (GObject *object, guint property_id, const GV
   EphyCompletionModel *self = EPHY_COMPLETION_MODEL (object);
 
   switch (property_id) {
-  case PROP_HISTORY_SERVICE:
-    self->priv->history_service = EPHY_HISTORY_SERVICE (g_value_get_pointer (value));
-    break;
-  case PROP_BOOKMARKS: {
-    EphyBookmarks *bookmarks = EPHY_BOOKMARKS (g_value_get_pointer (value));
+    case PROP_HISTORY_SERVICE:
+      self->history_service = EPHY_HISTORY_SERVICE (g_value_get_pointer (value));
+      break;
+    case PROP_BOOKMARKS: {
+      EphyBookmarks *bookmarks = EPHY_BOOKMARKS (g_value_get_pointer (value));
 
-    self->priv->bookmarks = ephy_bookmarks_get_bookmarks (bookmarks);
-    self->priv->smart_bookmarks = ephy_bookmarks_get_smart_bookmarks (bookmarks);
+      self->bookmarks = ephy_bookmarks_get_bookmarks (bookmarks);
+      self->smart_bookmarks = ephy_bookmarks_get_smart_bookmarks (bookmarks);
     }
     break;
-  case PROP_USE_MARKUP:
-    self->priv->use_markup = g_value_get_boolean (value);
-    break;
-  default:
-    G_OBJECT_WARN_INVALID_PROPERTY_ID (self, property_id, pspec);
-    break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (self, property_id, pspec);
+      break;
   }
 }
 
 static void
 ephy_completion_model_finalize (GObject *object)
 {
-  EphyCompletionModelPrivate *priv = EPHY_COMPLETION_MODEL (object)->priv;
+  EphyCompletionModel *model = EPHY_COMPLETION_MODEL (object);
 
-  if (priv->search_terms) {
-    free_search_terms (priv->search_terms);
-    priv->search_terms = NULL;
+  if (model->search_terms) {
+    free_search_terms (model->search_terms);
+    model->search_terms = NULL;
   }
 
-  if (priv->cancellable) {
-    g_cancellable_cancel (priv->cancellable);
-    g_clear_object (&priv->cancellable);
+  if (model->cancellable) {
+    g_cancellable_cancel (model->cancellable);
+    g_clear_object (&model->cancellable);
   }
 
   G_OBJECT_CLASS (ephy_completion_model_parent_class)->finalize (object);
@@ -127,34 +124,24 @@ ephy_completion_model_class_init (EphyCompletionModelClass *klass)
   object_class->constructed = ephy_completion_model_constructed;
   object_class->finalize = ephy_completion_model_finalize;
 
-  g_object_class_install_property (object_class,
-                                   PROP_HISTORY_SERVICE,
-                                   g_param_spec_pointer ("history-service",
-                                                         "History Service",
-                                                         "The history service",
-                                                         G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
-  g_object_class_install_property (object_class,
-                                   PROP_BOOKMARKS,
-                                   g_param_spec_pointer ("bookmarks",
-                                                         "Bookmarks",
-                                                         "The bookmarks",
-                                                         G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+  obj_properties[PROP_HISTORY_SERVICE] =
+    g_param_spec_pointer ("history-service",
+                          "History Service",
+                          "The history service",
+                          G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS);
 
-  g_object_class_install_property (object_class,
-                                   PROP_USE_MARKUP,
-                                   g_param_spec_boolean ("use-markup",
-                                                         "Whether we should be using markup",
-                                                         "Whether we should be using markup",
-                                                         TRUE,
-                                                         G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+  obj_properties[PROP_BOOKMARKS] =
+    g_param_spec_pointer ("bookmarks",
+                          "Bookmarks",
+                          "The bookmarks",
+                          G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS);
 
-  g_type_class_add_private (object_class, sizeof (EphyCompletionModelPrivate));
+  g_object_class_install_properties (object_class, LAST_PROP, obj_properties);
 }
 
 static void
 ephy_completion_model_init (EphyCompletionModel *model)
 {
-  model->priv = EPHY_COMPLETION_MODEL_GET_PRIVATE (model);
 }
 
 static gboolean
@@ -167,7 +154,7 @@ is_base_address (const char *address)
    * Neither scheme nor host contain a slash, so we can use slashes
    * figure out if it's a base address.
    *
-   * Note: previous code was using a GRegExp to do the same thing. 
+   * Note: previous code was using a GRegExp to do the same thing.
    * While regexps are much nicer to read, they're also a lot
    * slower.
    */
@@ -187,8 +174,8 @@ is_base_address (const char *address)
 
 static int
 get_relevance (const char *location,
-               int visit_count,
-               gboolean is_bookmark)
+               int         visit_count,
+               gboolean    is_bookmark)
 {
   /* FIXME: use frecency. */
   int relevance = 0;
@@ -205,7 +192,7 @@ get_relevance (const char *location,
     else
       relevance = visit_count;
   }
-  
+
   return relevance;
 }
 
@@ -227,7 +214,7 @@ icon_loaded_cb (GObject *source, GAsyncResult *result, gpointer user_data)
 {
   GtkTreeIter iter;
   GtkTreePath *path;
-  IconLoadData *data = (IconLoadData *) user_data;
+  IconLoadData *data = (IconLoadData *)user_data;
   WebKitFaviconDatabase *database = WEBKIT_FAVICON_DATABASE (source);
   GdkPixbuf *favicon = NULL;
   cairo_surface_t *icon_surface = webkit_favicon_database_get_favicon_finish (database, result, NULL);
@@ -253,54 +240,28 @@ icon_loaded_cb (GObject *source, GAsyncResult *result, gpointer user_data)
   g_slice_free (IconLoadData, data);
 }
 
-static gchar *
-get_row_text (const gchar *url, const gchar *title, const gchar *subtitle_color)
-{
-  gchar *unescaped_url;
-  gchar *text;
-
-  if (!url)
-    return g_markup_escape_text (title, -1);
-
-  unescaped_url = ephy_uri_safe_unescape (url);
-  if (g_strcmp0 (url, title) == 0)
-    text = g_markup_escape_text (unescaped_url, -1);
-  else
-    text = g_markup_printf_escaped ("%s\n<span font-size=\"small\" color=\"%s\">%s</span>", title, subtitle_color, unescaped_url);
-  g_free (unescaped_url);
-
-  return text;
-}
-
 static void
-set_row_in_model (EphyCompletionModel *model, int position, PotentialRow *row, const gchar *subtitle_color)
+set_row_in_model (EphyCompletionModel *model, int position, PotentialRow *row)
 {
   GtkTreeIter iter;
   GtkTreePath *path;
   IconLoadData *data;
-  WebKitFaviconDatabase* database;
-  gchar *text;
+  WebKitFaviconDatabase *database;
   EphyEmbedShell *shell = ephy_embed_shell_get_default ();
 
   database = webkit_web_context_get_favicon_database (ephy_embed_shell_get_web_context (shell));
 
-  if (model->priv->use_markup)
-    text = get_row_text (row->location, row->title, subtitle_color);
-  else
-    text = g_strdup (row->title);
-
   gtk_list_store_insert_with_values (GTK_LIST_STORE (model), &iter, position,
-                                     EPHY_COMPLETION_TEXT_COL, text ? text : "",
+                                     EPHY_COMPLETION_TEXT_COL, row->title ? row->title : "",
                                      EPHY_COMPLETION_URL_COL, row->location,
                                      EPHY_COMPLETION_ACTION_COL, row->location,
                                      EPHY_COMPLETION_KEYWORDS_COL, row->keywords ? row->keywords : "",
                                      EPHY_COMPLETION_EXTRA_COL, row->is_bookmark,
                                      EPHY_COMPLETION_RELEVANCE_COL, row->relevance,
                                      -1);
-  g_free (text);
 
   data = g_slice_new (IconLoadData);
-  data->model = GTK_LIST_STORE (g_object_ref(model));
+  data->model = GTK_LIST_STORE (g_object_ref (model));
   path = gtk_tree_model_get_path (GTK_TREE_MODEL (model), &iter);
   data->row_reference = gtk_tree_row_reference_new (GTK_TREE_MODEL (model), path);
   gtk_tree_path_free (path);
@@ -309,73 +270,41 @@ set_row_in_model (EphyCompletionModel *model, int position, PotentialRow *row, c
                                        NULL, icon_loaded_cb, data);
 }
 
-static gchar *
-get_text_column_subtitle_color (void)
-{
-  GtkWidgetPath *path;
-  GtkStyleContext *style_context;
-  GdkRGBA rgba;
-
-  path = gtk_widget_path_new ();
-  gtk_widget_path_prepend_type (path, GTK_TYPE_ENTRY);
-  gtk_widget_path_iter_add_class (path, 0, GTK_STYLE_CLASS_ENTRY);
-
-  style_context = gtk_style_context_new ();
-  gtk_style_context_set_path (style_context, path);
-  gtk_widget_path_free (path);
-
-  gtk_style_context_add_class (style_context, GTK_STYLE_CLASS_ENTRY);
-  gtk_style_context_get_color (style_context, GTK_STATE_FLAG_INSENSITIVE, &rgba);
-  g_object_unref (style_context);
-
-  return g_strdup_printf ("#%04X%04X%04X",
-                          (guint)(rgba.red * (gdouble)65535),
-                          (guint)(rgba.green * (gdouble)65535),
-                          (guint)(rgba.blue * (gdouble)65535));
-}
-
 static void
 replace_rows_in_model (EphyCompletionModel *model, GSList *new_rows)
 {
   /* This is by far the simplest way of doing, and yet it gives
    * basically the same result than the other methods... */
   int i;
-  gchar *subtitle_color = NULL;
 
   gtk_list_store_clear (GTK_LIST_STORE (model));
 
   if (!new_rows)
     return;
 
-  if (model->priv->use_markup)
-    subtitle_color = get_text_column_subtitle_color ();
-
   for (i = 0; new_rows != NULL; i++) {
-    PotentialRow *row = (PotentialRow*)new_rows->data;
+    PotentialRow *row = (PotentialRow *)new_rows->data;
 
-    set_row_in_model (model, i, row, subtitle_color);
+    set_row_in_model (model, i, row);
     new_rows = new_rows->next;
   }
-
-  g_free (subtitle_color);
 }
 
 static gboolean
 should_add_bookmark_to_model (EphyCompletionModel *model,
-                              const char *search_string,
-                              const char *title,
-                              const char *location,
-                              const char *keywords)
+                              const char          *search_string,
+                              const char          *title,
+                              const char          *location,
+                              const char          *keywords)
 {
   gboolean ret = TRUE;
-  EphyCompletionModelPrivate *priv = model->priv;
 
-  if (priv->search_terms) {
+  if (model->search_terms) {
     GSList *iter;
     GRegex *current = NULL;
 
-    for (iter = priv->search_terms; iter != NULL; iter = iter->next) {
-      current = (GRegex*)iter->data;
+    for (iter = model->search_terms; iter != NULL; iter = iter->next) {
+      current = (GRegex *)iter->data;
       if ((!g_regex_match (current, title ? title : "", G_REGEX_MATCH_NOTEMPTY, NULL)) &&
           (!g_regex_match (current, location ? location : "", G_REGEX_MATCH_NOTEMPTY, NULL)) &&
           (!g_regex_match (current, keywords ? keywords : "", G_REGEX_MATCH_NOTEMPTY, NULL))) {
@@ -399,7 +328,7 @@ static int
 find_url (gconstpointer a,
           gconstpointer b)
 {
-  return g_strcmp0 (((PotentialRow*)a)->location,
+  return g_strcmp0 (((PotentialRow *)a)->location,
                     ((char *)b));
 }
 
@@ -425,18 +354,18 @@ free_potential_row (PotentialRow *row)
   g_free (row->title);
   g_free (row->location);
   g_free (row->keywords);
-  
+
   g_slice_free (PotentialRow, row);
 }
 
 static GSList *
-add_to_potential_rows (GSList *rows,
+add_to_potential_rows (GSList     *rows,
                        const char *title,
                        const char *location,
                        const char *keywords,
-                       int visit_count,
-                       gboolean is_bookmark,
-                       gboolean search_for_duplicates)
+                       int         visit_count,
+                       gboolean    is_bookmark,
+                       gboolean    search_for_duplicates)
 {
   gboolean found = FALSE;
   PotentialRow *row = potential_row_new (title, location, keywords, visit_count, is_bookmark);
@@ -446,10 +375,10 @@ add_to_potential_rows (GSList *rows,
 
     p = g_slist_find_custom (rows, location, find_url);
     if (p) {
-      PotentialRow *match = (PotentialRow*)p->data;
+      PotentialRow *match = (PotentialRow *)p->data;
       if (row->relevance > match->relevance)
         match->relevance = row->relevance;
-      
+
       found = TRUE;
       free_potential_row (row);
     }
@@ -464,8 +393,8 @@ add_to_potential_rows (GSList *rows,
 static int
 sort_by_relevance (gconstpointer a, gconstpointer b)
 {
-  PotentialRow *r1 = (PotentialRow*)a;
-  PotentialRow *r2 = (PotentialRow*)b;
+  PotentialRow *r1 = (PotentialRow *)a;
+  PotentialRow *r2 = (PotentialRow *)b;
 
   if (r1->relevance < r2->relevance)
     return 1;
@@ -477,19 +406,18 @@ sort_by_relevance (gconstpointer a, gconstpointer b)
 
 static void
 query_completed_cb (EphyHistoryService *service,
-                    gboolean success,
-                    gpointer result_data,
-                    FindURLsData *user_data)
+                    gboolean            success,
+                    gpointer            result_data,
+                    FindURLsData       *user_data)
 {
   EphyCompletionModel *model = user_data->model;
-  EphyCompletionModelPrivate *priv = model->priv;
   GList *p, *urls;
   GPtrArray *children;
   GSList *list = NULL;
-  int i;
+  guint i;
 
   /* Bookmarks */
-  children = ephy_node_get_children (priv->bookmarks);
+  children = ephy_node_get_children (model->bookmarks);
 
   /* FIXME: perhaps this could be done in a service thread? There
    * should never be a ton of bookmarks, but seems a bit cleaner and
@@ -503,7 +431,7 @@ query_completed_cb (EphyHistoryService *service,
     location = ephy_node_get_property_string (kid, EPHY_NODE_BMK_PROP_LOCATION);
     title = ephy_node_get_property_string (kid, EPHY_NODE_BMK_PROP_TITLE);
     keywords = ephy_node_get_property_string (kid, EPHY_NODE_BMK_PROP_KEYWORDS);
-    is_smart = ephy_node_has_child (priv->smart_bookmarks, kid);
+    is_smart = ephy_node_has_child (model->smart_bookmarks, kid);
 
     /* Smart bookmarks are already added to the completion menu as completion actions */
     if (!is_smart && should_add_bookmark_to_model (model, user_data->search_string,
@@ -512,10 +440,10 @@ query_completed_cb (EphyHistoryService *service,
   }
 
   /* History */
-  urls = (GList*)result_data;
+  urls = (GList *)result_data;
 
   for (p = urls; p != NULL; p = p->next) {
-    EphyHistoryURL *url = (EphyHistoryURL*)p->data;
+    EphyHistoryURL *url = (EphyHistoryURL *)p->data;
 
     list = add_to_potential_rows (list, url->title, url->url, NULL, url->visit_count, FALSE, TRUE);
   }
@@ -535,12 +463,12 @@ query_completed_cb (EphyHistoryService *service,
   g_slice_free (FindURLsData, user_data);
   g_list_free_full (urls, (GDestroyNotify)ephy_history_url_free);
   g_slist_free_full (list, (GDestroyNotify)free_potential_row);
-  g_clear_object (&priv->cancellable);
+  g_clear_object (&model->cancellable);
 }
 
 static void
 update_search_terms (EphyCompletionModel *model,
-                     const char *text)
+                     const char          *text)
 {
   const char *current;
   const char *ptr;
@@ -550,16 +478,15 @@ update_search_terms (EphyCompletionModel *model,
   GRegex *quote_regex;
   gint count;
   gboolean inside_quotes = FALSE;
-  EphyCompletionModelPrivate *priv = model->priv;
 
-  if (priv->search_terms) {
-    free_search_terms (priv->search_terms);
-    priv->search_terms = NULL;
+  if (model->search_terms) {
+    free_search_terms (model->search_terms);
+    model->search_terms = NULL;
   }
 
   quote_regex = g_regex_new ("\"", G_REGEX_OPTIMIZE,
                              G_REGEX_MATCH_NOTEMPTY, NULL);
-    
+
   /*
    * This code loops through the string using pointer arythmetics.
    * Although the string we are handling may contain UTF-8 chars
@@ -568,7 +495,7 @@ update_search_terms (EphyCompletionModel *model,
    */
   for (count = 0, current = ptr = text; ptr[0] != '\0'; ptr++, count++) {
     /*
-     * If we found a double quote character; we will 
+     * If we found a double quote character; we will
      * consume bytes up until the next quote, or
      * end of line;
      */
@@ -590,7 +517,7 @@ update_search_terms (EphyCompletionModel *model,
        */
       if (ptr[1] == '\0')
         count++;
-        
+
       /*
        * remove quotes, and quote any regex-sensitive
        * characters
@@ -606,7 +533,7 @@ update_search_terms (EphyCompletionModel *model,
         term_regex = g_regex_new (term,
                                   G_REGEX_CASELESS | G_REGEX_OPTIMIZE,
                                   G_REGEX_MATCH_NOTEMPTY, NULL);
-        priv->search_terms = g_slist_append (priv->search_terms, term_regex);
+        model->search_terms = g_slist_append (model->search_terms, term_regex);
       }
       g_free (term);
 
@@ -622,12 +549,11 @@ update_search_terms (EphyCompletionModel *model,
 #define MAX_COMPLETION_HISTORY_URLS 8
 
 void
-ephy_completion_model_update_for_string (EphyCompletionModel *model,
-                                         const char *search_string,
+ephy_completion_model_update_for_string (EphyCompletionModel   *model,
+                                         const char            *search_string,
                                          EphyHistoryJobCallback callback,
-                                         gpointer data)
+                                         gpointer               data)
 {
-  EphyCompletionModelPrivate *priv;
   char **strings;
   int i;
   GList *query = NULL;
@@ -636,8 +562,6 @@ ephy_completion_model_update_for_string (EphyCompletionModel *model,
   g_return_if_fail (EPHY_IS_COMPLETION_MODEL (model));
   g_return_if_fail (search_string != NULL);
 
-  priv = model->priv;
-
   /* Split the search string. */
   strings = g_strsplit (search_string, " ", -1);
   for (i = 0; strings[i]; i++)
@@ -652,26 +576,25 @@ ephy_completion_model_update_for_string (EphyCompletionModel *model,
   user_data->callback = callback;
   user_data->user_data = data;
 
-  if (priv->cancellable) {
-    g_cancellable_cancel (priv->cancellable);
-    g_object_unref (priv->cancellable);
+  if (model->cancellable) {
+    g_cancellable_cancel (model->cancellable);
+    g_object_unref (model->cancellable);
   }
-  priv->cancellable = g_cancellable_new ();
+  model->cancellable = g_cancellable_new ();
 
-  ephy_history_service_find_urls (priv->history_service,
+  ephy_history_service_find_urls (model->history_service,
                                   0, 0,
                                   MAX_COMPLETION_HISTORY_URLS, 0,
                                   query,
                                   EPHY_HISTORY_SORT_MOST_VISITED,
-                                  priv->cancellable,
+                                  model->cancellable,
                                   (EphyHistoryJobCallback)query_completed_cb,
                                   user_data);
 }
 
 EphyCompletionModel *
 ephy_completion_model_new (EphyHistoryService *history_service,
-                           EphyBookmarks      *bookmarks,
-                           gboolean            use_markup)
+                           EphyBookmarks      *bookmarks)
 {
   g_return_val_if_fail (EPHY_IS_HISTORY_SERVICE (history_service), NULL);
   g_return_val_if_fail (EPHY_IS_BOOKMARKS (bookmarks), NULL);
@@ -679,6 +602,5 @@ ephy_completion_model_new (EphyHistoryService *history_service,
   return g_object_new (EPHY_TYPE_COMPLETION_MODEL,
                        "history-service", history_service,
                        "bookmarks", bookmarks,
-                       "use-markup", use_markup,
                        NULL);
 }
