@@ -13,9 +13,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with Epiphany; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA  02110-1301  USA
+ * along with Epiphany.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -30,47 +28,47 @@
 static void
 test_ephy_completion_model_create (void)
 {
-    EphyCompletionModel *model;
-    model = ephy_completion_model_new (EPHY_HISTORY_SERVICE (ephy_embed_shell_get_global_history_service (ephy_embed_shell_get_default ())),
-                                       ephy_shell_get_bookmarks (ephy_shell_get_default ()), TRUE);
-    g_assert (model);
-    g_object_unref (model);
+  EphyCompletionModel *model;
+  model = ephy_completion_model_new (EPHY_HISTORY_SERVICE (ephy_embed_shell_get_global_history_service (ephy_embed_shell_get_default ())),
+                                     ephy_shell_get_bookmarks (ephy_shell_get_default ()));
+  g_assert (model);
+  g_object_unref (model);
 }
 
 static void
 update_empty_cb (EphyHistoryService *service,
-                 gboolean success,
-                 gpointer result_data,
-                 GMainLoop *loop)
+                 gboolean            success,
+                 gpointer            result_data,
+                 GMainLoop          *loop)
 {
-    GList *results = (GList*)result_data;
+  GList *results = (GList *)result_data;
 
-    g_assert (success);
-    g_assert (results == NULL);
+  g_assert (success);
+  g_assert (results == NULL);
 
-    g_main_loop_quit (loop);
+  g_main_loop_quit (loop);
 }
 
 static void
 test_ephy_completion_model_update_empty (void)
 {
-    EphyCompletionModel *model;
-    GMainLoop *loop = NULL;
+  EphyCompletionModel *model;
+  GMainLoop *loop = NULL;
 
-    model = ephy_completion_model_new (EPHY_HISTORY_SERVICE (ephy_embed_shell_get_global_history_service (ephy_embed_shell_get_default ())),
-                                       ephy_shell_get_bookmarks (ephy_shell_get_default ()), TRUE);
-    g_assert (model);
+  model = ephy_completion_model_new (EPHY_HISTORY_SERVICE (ephy_embed_shell_get_global_history_service (ephy_embed_shell_get_default ())),
+                                     ephy_shell_get_bookmarks (ephy_shell_get_default ()));
+  g_assert (model);
 
-    loop = g_main_loop_new (NULL, FALSE);
+  loop = g_main_loop_new (NULL, FALSE);
 
-    ephy_completion_model_update_for_string (model, "hello",
-                                             (EphyHistoryJobCallback)update_empty_cb,
-                                             loop);
+  ephy_completion_model_update_for_string (model, "hello",
+                                           (EphyHistoryJobCallback)update_empty_cb,
+                                           loop);
 
-    g_main_loop_run (loop);
+  g_main_loop_run (loop);
 
-    g_object_unref (model);
-    g_main_loop_unref (loop);
+  g_object_unref (model);
+  g_main_loop_unref (loop);
 }
 
 int
