@@ -17,9 +17,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with Epiphany; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA  02110-1301  USA
+ * along with Epiphany.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -128,7 +126,7 @@ test_is_empty (const IsEmptyTest *test)
 int
 main (int argc, char *argv[])
 {
-  int i;
+  guint i;
   gtk_test_init (&argc, &argv);
 
   for (i = 0; i < G_N_ELEMENTS (tests_has_scheme); i++) {
@@ -140,7 +138,7 @@ main (int argc, char *argv[])
                              test.name, NULL);
 
     g_test_add_data_func (test_name, test.test,
-                          (GTestDataFunc) test_address_has_web_scheme);
+                          (GTestDataFunc)test_address_has_web_scheme);
 
     g_free (test_name);
   }
@@ -154,7 +152,7 @@ main (int argc, char *argv[])
                              test.name, NULL);
 
     g_test_add_data_func (test_name, test.test,
-                          (GTestDataFunc) test_address_no_web_scheme);
+                          (GTestDataFunc)test_address_no_web_scheme);
 
     g_free (test_name);
   }
